@@ -29,7 +29,8 @@ l_int32 main(int    argc,
 {
 l_int32       i, n;
 l_float32     a, b, c;
-L_DEWARP     *dew, *dew2;
+L_DEWARP     *dew1, *dew2, *dew3, *dew4;
+L_DEWARPA    *dewa1, *dewa2;
 DPIX         *dpix1, *dpix2, *dpix3;
 FPIX         *fpix1, *fpix2, *fpix3;
 NUMA         *nax, *nafit;
@@ -42,19 +43,18 @@ L_REGPARAMS  *rp;
     if (regTestSetup(argc, argv, &rp))
               return 1;
 
+        /* Read page 7, normalize for varying background and binarize */
     pixs = pixRead("1555-7.jpg");
-    
-        /* Normalize for varying background and binarize */
     pixn = pixBackgroundNormSimple(pixs, NULL, NULL);
     pixg = pixConvertRGBToGray(pixn, 0.5, 0.3, 0.2);
     pixb = pixThresholdToBinary(pixg, 130);
     pixDestroy(&pixn);
     pixDestroy(&pixg);
     regTestWritePixAndCheck(rp, pixb, IFF_PNG);  /* 0 */
-    pixDisplayWithTitle(pixb, 0, 0, "binarized input", rp->display);
+    pixDisplayWithTitle(pixb, 0, 0, "page 7 binarized input", rp->display);
 
         /* Get the textline centers */
-    ptaa1 = pixGetTextlineCenters(pixb, 0);
+    ptaa1 = dewarpGetTextlineCenters(pixb, 0);
     pixt1 = pixCreateTemplate(pixs);
     pixt2 = pixDisplayPtaa(pixt1, ptaa1);
     regTestWritePixAndCheck(rp, pixt2, IFF_PNG);  /* 1 */
@@ -62,7 +62,7 @@ L_REGPARAMS  *rp;
     pixDestroy(&pixt1);
 
         /* Remove short lines */
-    ptaa2 = ptaaRemoveShortLines(pixb, ptaa1, 0.8, 0);
+    ptaa2 = dewarpRemoveShortLines(pixb, ptaa1, 0.8, 0);
 
         /* Fit to quadratic */
     n = ptaaGetCount(ptaa2);
@@ -84,75 +84,72 @@ L_REGPARAMS  *rp;
     ptaaDestroy(&ptaa2);
     pixDestroy(&pixt2);
 
-        /* Run with only vertical disparity correction */
-    if ((dew = dewarpCreate(pixb, 7, 30, 15, 0)) == NULL)
+        /* Build the model for page 7 and dewarp */
+    dewa1 = dewarpaCreate(2, 30, 1, 15, 30);
+    if ((dew1 = dewarpCreate(pixb, 7)) == NULL)
         return ERROR_INT("\n\n\n FAILURE !!! \n\n\n", rp->testname, 1);
-    dewarpBuildModel(dew, 0);
-    dewarpApplyDisparity(dew, pixb, 0);
-    regTestWritePixAndCheck(rp, dew->pixd, IFF_PNG);  /* 3 */
-    pixDisplayWithTitle(dew->pixd, 400, 0, "fixed for vert disparity",
-                        rp->display);
-    dewarpDestroy(&dew);
+    dewarpaInsertDewarp(dewa1, dew1);
+    dewarpBuildModel(dew1, NULL);
+    dewarpaApplyDisparity(dewa1, 7, pixb, NULL);
+    regTestWritePixAndCheck(rp, dew1->pixd, IFF_PNG);  /* 3 */
+    pixDisplayWithTitle(dew1->pixd, 400, 0, "page 7 dewarped", rp->display);
 
-        /* Run with both vertical and horizontal disparity correction */
-    if ((dew = dewarpCreate(pixb, 7, 30, 15, 1)) == NULL)
-        return ERROR_INT("\n\n\n FAILURE !!! \n\n\n", rp->testname, 1);
-    dewarpBuildModel(dew, 0);
-    dewarpApplyDisparity(dew, pixb, 0);
-    regTestWritePixAndCheck(rp, dew->pixd, IFF_PNG);  /* 4 */
-    pixDisplayWithTitle(dew->pixd, 800, 0, "fixed for both disparities",
-                        rp->display);
-
-        /* Read another image, normalize background and binarize */
+        /* Read page 3, normalize background and binarize */
     pixs2 = pixRead("1555-3.jpg");
     pixn2 = pixBackgroundNormSimple(pixs2, NULL, NULL);
     pixg2 = pixConvertRGBToGray(pixn2, 0.5, 0.3, 0.2);
     pixb2 = pixThresholdToBinary(pixg2, 130);
     pixDestroy(&pixn2);
     pixDestroy(&pixg2);
-    regTestWritePixAndCheck(rp, pixb, IFF_PNG);  /* 5 */
+    regTestWritePixAndCheck(rp, pixb, IFF_PNG);  /* 4 */
     pixDisplayWithTitle(pixb, 0, 400, "binarized input (2)", rp->display);
 
-        /* Minimize and re-apply previous disparity to this image */
-    dewarpMinimize(dew);
-    dewarpApplyDisparity(dew, pixb2, 0);
-    regTestWritePixAndCheck(rp, dew->pixd, IFF_PNG);  /* 6 */
-    pixDisplayWithTitle(dew->pixd, 400, 400, "fixed (2) for both disparities",
-                        rp->display);
+        /* Minimize and re-apply page 7 disparity to this image */
+    dewarpaInsertRefModels(dewa1, 0);
+    dewarpaApplyDisparity(dewa1, 3, pixb2, NULL);
+    dew2 = dewarpaGetDewarp(dewa1, 3);
+    regTestWritePixAndCheck(rp, dew2->pixd, IFF_PNG);  /* 5 */
+    pixDisplayWithTitle(dew2->pixd, 400, 400, "page 3 dewarped", rp->display);
 
         /* Write and read back minimized dewarp struct */
-    dewarpWrite("/tmp/dewarp.7.dew", dew);
+    dewarpMinimize(dew1);
+    dewarpWrite("/tmp/dewarp.6.dew", dew1);
+    regTestCheckFile(rp, "/tmp/dewarp.6.dew");  /* 6 */
+    dew3 = dewarpRead("/tmp/dewarp.6.dew");
+    dewarpWrite("/tmp/dewarp.7.dew", dew3);
     regTestCheckFile(rp, "/tmp/dewarp.7.dew");  /* 7 */
-    dew2 = dewarpRead("/tmp/dewarp.7.dew");
-    dewarpWrite("/tmp/dewarp.8.dew", dew2);
-    regTestCheckFile(rp, "/tmp/dewarp.8.dew");  /* 8 */
-    regTestCompareFiles(rp, 7, 8);  /* 9 */
+    regTestCompareFiles(rp, 6, 7);  /* 8 */
 
-        /* Apply dew2 to pixb2 */
-    dewarpApplyDisparity(dew2, pixb2, 0);
-    regTestWritePixAndCheck(rp, dew2->pixd, IFF_PNG);  /* 10 */
-    pixDisplayWithTitle(dew->pixd, 800, 400, "fixed (3) for both disparities",
+        /* Apply this minimized dew to page 3 in a new dewa */
+    dewa2 = dewarpaCreate(2, 30, 1, 15, 30);
+    dewarpaInsertDewarp(dewa2, dew3);
+    dewarpaInsertRefModels(dewa2, 0);
+    dewarpaListPages(dewa2);  /* just for fun: should be 1, 3, 5, 7 */
+    dewarpaApplyDisparity(dewa2, 3, pixb2, NULL);
+    dew4 = dewarpaGetDewarp(dewa2, 3);
+    regTestWritePixAndCheck(rp, dew4->pixd, IFF_PNG);  /* 9 */
+    pixDisplayWithTitle(dew4->pixd, 800, 400, "page 3 dewarped again",
                         rp->display);
 
-        /* Minimize, repopulate disparity arrays, and apply again */
-    dewarpMinimize(dew2);
-    dewarpApplyDisparity(dew2, pixb2, 0);
-    regTestWritePixAndCheck(rp, dew2->pixd, IFF_PNG);  /* 11 */
-    regTestCompareFiles(rp, 10, 11);  /* 12 */
-    pixDisplayWithTitle(dew->pixd, 900, 400, "fixed (4) for both disparities",
+        /* Minimize, re-populate disparity arrays, and apply again */
+    dewarpMinimize(dew3);
+    dewarpaApplyDisparity(dewa2, 3, pixb2, NULL);
+    regTestWritePixAndCheck(rp, dew4->pixd, IFF_PNG);  /* 10 */
+    regTestCompareFiles(rp, 9, 10);  /* 11 */
+    pixDisplayWithTitle(dew3->pixd, 900, 400, "page 3 dewarped yet again",
                         rp->display);
 
         /* Test a few of the fpix functions */
-    fpix1 = fpixClone(dew->sampvdispar);
-    fpixWrite("/tmp/sampv.13.fpix", fpix1);
+    fpix1 = fpixClone(dew3->sampvdispar);
+    fpixWrite("/tmp/sampv.12.fpix", fpix1);
+    regTestCheckFile(rp, "/tmp/sampv.12.fpix");  /* 12 */
+    fpix2 = fpixRead("/tmp/sampv.12.fpix");
+    fpixWrite("/tmp/sampv.13.fpix", fpix2);
     regTestCheckFile(rp, "/tmp/sampv.13.fpix");  /* 13 */
-    fpix2 = fpixRead("/tmp/sampv.13.fpix");
-    fpixWrite("/tmp/sampv.14.fpix", fpix2);
-    regTestCheckFile(rp, "/tmp/sampv.14.fpix");  /* 14 */
-    regTestCompareFiles(rp, 13, 14);  /* 15 */
+    regTestCompareFiles(rp, 12, 13);  /* 14 */
     fpix3 = fpixScaleByInteger(fpix2, 30);
-    pixt1 = fpixRenderContours(fpix3, -2., 2.0, 0.2);
-    regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 16 */
+    pixt1 = fpixRenderContours(fpix3, 2.0, 0.2);
+    regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 15 */
     pixDisplayWithTitle(pixt1, 0, 800, "v. disparity contours", rp->display);
     fpixDestroy(&fpix1);
     fpixDestroy(&fpix2);
@@ -160,32 +157,30 @@ L_REGPARAMS  *rp;
     pixDestroy(&pixt1);
 
         /* Test a few of the dpix functions */
-    dpix1 = fpixConvertToDPix(dew->sampvdispar);
-    dpixWrite("/tmp/sampv.17.dpix", dpix1);
+    dpix1 = fpixConvertToDPix(dew3->sampvdispar);
+    dpixWrite("/tmp/sampv.16.dpix", dpix1);
+    regTestCheckFile(rp, "/tmp/sampv.16.dpix");  /* 16 */
+    dpix2 = dpixRead("/tmp/sampv.16.dpix");
+    dpixWrite("/tmp/sampv.17.dpix", dpix2);
     regTestCheckFile(rp, "/tmp/sampv.17.dpix");  /* 17 */
-    dpix2 = dpixRead("/tmp/sampv.17.dpix");
-    dpixWrite("/tmp/sampv.18.dpix", dpix2);
-    regTestCheckFile(rp, "/tmp/sampv.18.dpix");  /* 18 */
-    regTestCompareFiles(rp, 17, 18);  /* 19 */
+    regTestCompareFiles(rp, 16, 17);  /* 18 */
     dpix3 = dpixScaleByInteger(dpix2, 30);
     fpix3 = dpixConvertToFPix(dpix3);
-    pixt1 = fpixRenderContours(fpix3, -2., 2.0, 0.2);
-    regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 20 */
+    pixt1 = fpixRenderContours(fpix3, 2.0, 0.2);
+    regTestWritePixAndCheck(rp, pixt1, IFF_PNG);  /* 19 */
     pixDisplayWithTitle(pixt1, 400, 800, "v. disparity contours", rp->display);
-    regTestCompareFiles(rp, 16, 20);  /* 21 */
+    regTestCompareFiles(rp, 15, 19);  /* 20 */
     dpixDestroy(&dpix1);
     dpixDestroy(&dpix2);
     dpixDestroy(&dpix3);
     fpixDestroy(&fpix3);
     pixDestroy(&pixt1);
 
-    dewarpDestroy(&dew);
-    dewarpDestroy(&dew2);
+    dewarpaDestroy(&dewa1);
+    dewarpaDestroy(&dewa2);
     pixDestroy(&pixs);
     pixDestroy(&pixb);
     pixDestroy(&pixs2);
     pixDestroy(&pixb2);
-    regTestCleanup(rp);
-    return 0;
+    return regTestCleanup(rp);
 }
-
