@@ -77,7 +77,7 @@ static char  mainName[] = "fileinfo";
         tiffGetCount(fp, &npages);
         lept_fclose(fp);
         if (npages == 1)
-            fprintf(stderr, "    One page in file\n", npages);
+            fprintf(stderr, "    One page in file\n");
         else
             fprintf(stderr, "    %d pages in file\n", npages);
         fprintTiffInfo(stderr, filein);
