@@ -454,8 +454,7 @@ finish:
         fprintf(stderr,
             "  ******* Failure on at least one test *******\n\n");
 
-    regTestCleanup(rp);
-    return 0;
+    return regTestCleanup(rp);
 }
 
 
@@ -695,5 +694,3 @@ get_tiff_compression_name(char    *buf,
         fprintf(stderr, "format %d: not tiff\n", format);
     return;
 }
-
-
