@@ -379,7 +379,7 @@ static char  mainName[] = "pdfiotest";
     pixWrite("/tmp/image/file4.jpg", pix6, IFF_JFIF_JPEG);
 
     startTimer();
-    convertFilesToPdf("/tmp/image", "file", 100, 0.8, 75, "4 file test",
+    convertFilesToPdf("/tmp/image", "file", 100, 0.8, 0, 75, "4 file test",
                       "/tmp/fourimages.pdf");
     fprintf(stderr, "Time: %7.3f\n", stopTimer());
     pixDestroy(&pix1);
@@ -443,5 +443,3 @@ PIX  *pix1, *pix2, *pixd;
     pixDestroy(&pix2);
     return pixd;
 }
-
-
