@@ -151,8 +151,7 @@ L_REGPARAMS  *rp;
             "  ******* Failure on at least one test *******\n\n");
 
     if (failure) rp->success = FALSE;
-    regTestCleanup(rp);
-    return 0;
+    return regTestCleanup(rp);
 }
 
 
@@ -243,4 +242,3 @@ size_t    nbytes1, nbytes2;
 
     return ret1 || ret2;
 }
-
