@@ -50,8 +50,7 @@ L_REGPARAMS  *rp;
     TestProjection(rp, pixg2);
     pixDestroy(&pixg1);
     pixDestroy(&pixg2);
-    regTestCleanup(rp);
-    return 0;
+    return regTestCleanup(rp);
 }
 
 
@@ -181,5 +180,3 @@ PIXA    *pixa;
     numaDestroy(&na12);
     return;
 }
-
-
