@@ -139,8 +139,7 @@ L_REGPARAMS  *rp;
 
     pixDestroy(&pixs);
     lept_free(array);
-    regTestCleanup(rp);
-    return 0;
+    return regTestCleanup(rp);
 }
 
 
@@ -215,4 +214,3 @@ PIXA   *pixa;
     pixDestroy(&pixt);
     return pixa;
 }
-
