@@ -38,9 +38,6 @@
  *            void       absDifferenceLow()
  */
 
-
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
@@ -526,7 +523,8 @@ absDifferenceLow(l_uint32  *datad,
                  l_uint32  *datas1,
                  l_uint32  *datas2,
                  l_int32    d,
-                 l_int32    wpls)
+                 l_int32    wpls1,
+                 l_int32    wpls2)
 {
 l_int32    i, j, val1, val2, diff;
 l_uint32   word1, word2;
@@ -538,8 +536,8 @@ l_uint32  *lines1, *lines2, *lined, *pdword;
     {
     case 8:
         for (i = 0; i < h; i++) {
-            lines1 = datas1 + i * wpls;
-            lines2 = datas2 + i * wpls;
+            lines1 = datas1 + i * wpls1;
+            lines2 = datas2 + i * wpls2;
             lined = datad + i * wpld;
             for (j = 0; j < w; j++) {
                 val1 = GET_DATA_BYTE(lines1, j);
@@ -551,8 +549,8 @@ l_uint32  *lines1, *lines2, *lined, *pdword;
         break;
     case 16:
         for (i = 0; i < h; i++) {
-            lines1 = datas1 + i * wpls;
-            lines2 = datas2 + i * wpls;
+            lines1 = datas1 + i * wpls1;
+            lines2 = datas2 + i * wpls2;
             lined = datad + i * wpld;
             for (j = 0; j < w; j++) {
                 val1 = GET_DATA_TWO_BYTES(lines1, j);
@@ -564,8 +562,8 @@ l_uint32  *lines1, *lines2, *lined, *pdword;
         break;
     case 32:
         for (i = 0; i < h; i++) {
-            lines1 = datas1 + i * wpls;
-            lines2 = datas2 + i * wpls;
+            lines1 = datas1 + i * wpls1;
+            lines2 = datas2 + i * wpls2;
             lined = datad + i * wpld;
             for (j = 0; j < w; j++) {
                 word1 = lines1[j];
@@ -593,4 +591,3 @@ l_uint32  *lines1, *lines2, *lined, *pdword;
 
     return;
 }
-
