@@ -31,6 +31,7 @@
  *           l_int32   boxSetGeometry()
  *           l_int32   boxGetRefcount()
  *           l_int32   boxChangeRefcount()
+ *           l_int32   boxIsValid()
  *
  *      Boxa creation, copy, destruction
  *           BOXA     *boxaCreate()
@@ -64,13 +65,17 @@
  *      Boxaa array extension
  *           l_int32   boxaaAddBoxa()
  *           l_int32   boxaaExtendArray()
+ *           l_int32   boxaaExtendArrayToSize()
  *
  *      Boxaa accessors
  *           l_int32   boxaaGetCount()
  *           l_int32   boxaaGetBoxCount()
  *           BOXA     *boxaaGetBoxa()
+ *           BOX      *boxaaGetBox()
  *
  *      Boxa array modifiers
+ *           l_int32   boxaaInitFull()
+ *           l_int32   boxaaExtendWithInit()
  *           l_int32   boxaaReplaceBoxa()
  *           l_int32   boxaaInsertBoxa()
  *           l_int32   boxaaRemoveBoxa()
@@ -345,6 +350,31 @@ boxChangeRefcount(BOX     *box,
 }
 
 
+/*!
+ *  boxIsValid()
+ *
+ *      Input:  box
+ *              &valid (<return> 1 if valid; 0 otherwise)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+boxIsValid(BOX      *box,
+           l_int32  *pvalid)
+{
+    PROCNAME("boxIsValid");
+
+    if (!pvalid)
+        return ERROR_INT("&valid not defined", procName, 1);
+    *pvalid = 0;
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+
+    if (box->w > 0 && box->h > 0)
+        *pvalid = 1;
+    return 0;
+}
+
+
 /*---------------------------------------------------------------------*
  *             Boxa creation, destruction, copy, extension             *
  *---------------------------------------------------------------------*/
@@ -720,7 +750,7 @@ BOX  *box;
  *
  *  Notes:
  *      (1) In-place replacement of one box.
- *      (2) The previous box at that location is destroyed.
+ *      (2) The previous box at that location, if any, is destroyed.
  */
 l_int32
 boxaReplaceBox(BOXA    *boxa,
@@ -1081,6 +1111,36 @@ boxaaExtendArray(BOXAA  *baa)
 }
 
 
+/*!
+ *  boxaaExtendArrayToSize()
+ *
+ *      Input:  boxaa
+ *              size (new size of boxa array)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) If necessary, reallocs the boxa ptr array to @size.
+ */
+l_int32
+boxaaExtendArrayToSize(BOXAA   *baa,
+                       l_int32  size)
+{
+    PROCNAME("boxaaExtendArrayToSize");
+
+    if (!baa)
+        return ERROR_INT("baa not defined", procName, 1);
+
+    if (size > baa->nalloc) {
+        if ((baa->boxa = (BOXA **)reallocNew((void **)&baa->boxa,
+                                             sizeof(BOXA *) * baa->nalloc,
+                                             size * sizeof(BOXA *))) == NULL)
+            return ERROR_INT("new ptr array not returned", procName, 1);
+        baa->nalloc = size;
+    }
+    return 0;
+}
+
+
 /*----------------------------------------------------------------------*
  *                           Boxaa accessors                            *
  *----------------------------------------------------------------------*/
@@ -1158,6 +1218,129 @@ l_int32  n;
 }
 
 
+/*!
+ *  boxaaGetBox()
+ *
+ *      Input:  baa
+ *              iboxa  (index into the boxa array in the boxaa)
+ *              ibox  (index into the box array in the boxa)
+ *              accessflag   (L_COPY or L_CLONE)
+ *      Return: box, or null on error
+ */
+BOX *
+boxaaGetBox(BOXAA   *baa,
+            l_int32  iboxa,
+            l_int32  ibox,
+            l_int32  accessflag)
+{
+BOX   *box;
+BOXA  *boxa;
+
+    PROCNAME("boxaaGetBox");
+
+    if ((boxa = boxaaGetBoxa(baa, iboxa, L_CLONE)) == NULL)
+        return (BOX *)ERROR_PTR("boxa not retrieved", procName, NULL);
+    if ((box = boxaGetBox(boxa, ibox, accessflag)) == NULL)
+        L_ERROR("box not retrieved", procName);
+    boxaDestroy(&boxa);
+    return box;
+}
+
+
+/*----------------------------------------------------------------------*
+ *                           Boxaa array modifiers                      *
+ *----------------------------------------------------------------------*/
+/*!
+ *  boxaaInitFull()
+ *
+ *      Input:  boxaa (typically empty)
+ *              boxa (to be replicated into the entire ptr array)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This initializes a boxaa by filling up the entire boxa ptr array
+ *          with copies of @boxa.  Any existing boxa are destroyed.
+ *          After this operation, the number of boxa is equal to
+ *          the number of allocated ptrs.
+ *      (2) Note that we use boxaaReplaceBox() instead of boxaInsertBox().
+ *          They both have the same effect when inserting into a NULL ptr
+ *          in the boxa ptr array
+ *      (3) Example usage.  This function is useful to prepare for a
+ *          random insertion (or replacement) of boxa into a boxaa.
+ *          To randomly insert boxa into a boxaa, up to some index "max":
+ *             Boxaa *baa = boxaaCreate(max);
+ *               // initialize the boxa
+ *             Boxa *boxa = boxaCreate(...);
+ *             ...  [optionally fix with boxes]
+ *             boxaaInitFull(baa, boxa);
+ *          A typical use is to initialize the array with empty boxa,
+ *          and to replace only a subset that must be aligned with
+ *          something else, such as a pixa.
+ */
+l_int32
+boxaaInitFull(BOXAA  *baa,
+              BOXA   *boxa)
+{
+l_int32  i, n;
+BOXA    *boxat;
+
+    PROCNAME("boxaaInitFull");
+
+    if (!baa)
+        return ERROR_INT("baa not defined", procName, 1);
+    if (!boxa)
+        return ERROR_INT("boxa not defined", procName, 1);
+
+    n = baa->nalloc;
+    baa->n = n;
+    for (i = 0; i < n; i++) {
+        boxat = boxaCopy(boxa, L_COPY);
+        boxaaReplaceBoxa(baa, i, boxat);
+    }
+    return 0;
+}
+
+
+/*!
+ *  boxaaExtendWithInit()
+ *
+ *      Input:  boxaa
+ *              maxindex
+ *              boxa (to be replicated into the extended ptr array)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This should be used on an existing boxaa that has been
+ *          fully loaded with boxa.  It then extends the boxaa,
+ *          loading all the additional ptrs with copies of boxa.
+ *          Typically, boxa will be empty.
+ */
+l_int32
+boxaaExtendWithInit(BOXAA   *baa,
+                    l_int32  maxindex,
+                    BOXA    *boxa)
+{
+l_int32  i, n;
+
+    PROCNAME("boxaaExtendWithInit");
+
+    if (!baa)
+        return ERROR_INT("baa not defined", procName, 1);
+    if (!boxa)
+        return ERROR_INT("boxa not defined", procName, 1);
+
+        /* Extend the ptr array if necessary */
+    n = boxaaGetCount(baa);
+    if (maxindex < n) return 0;
+    boxaaExtendArrayToSize(baa, maxindex + 1);
+
+        /* Fill the new entries with copies of boxa */
+    for (i = n; i <= maxindex; i++)
+        boxaaAddBoxa(baa, boxa, L_COPY);
+    return 0;
+}
+
+
 /*!
  *  boxaaReplaceBoxa()
  *
@@ -1633,4 +1816,3 @@ boxPrintStreamInfo(FILE  *fp,
 
     return 0;
 }
-
