@@ -66,6 +66,7 @@
  *           l_int32     pixcmapGammaTRC()
  *           l_int32     pixcmapContrastTRC()
  *           l_int32     pixcmapShiftIntensity()
+ *           l_int32     pixcmapShiftByComponent()
  */
 
 #include <string.h>
@@ -1554,9 +1555,9 @@ char     buf[4];
  *      Return: 0 if OK; 1 on error
  *
  *  Notes:
- *      - in-place transform
- *      - see pixGammaTRC() and numaGammaTRC() in enhance.c for
- *        description and use of transform
+ *      (1) This is an in-place transform
+ *      (2) See pixGammaTRC() and numaGammaTRC() in enhance.c
+ *          for description and use of transform
  */
 l_int32
 pixcmapGammaTRC(PIXCMAP   *cmap,
@@ -1564,8 +1565,8 @@ pixcmapGammaTRC(PIXCMAP   *cmap,
                 l_int32    minval,
                 l_int32    maxval)
 {
-l_int32   rval, gval, bval, trval, tgval, tbval, i, ncolors;
-NUMA     *nag;
+l_int32  rval, gval, bval, trval, tgval, tbval, i, ncolors;
+NUMA    *nag;
 
     PROCNAME("pixcmapGammaTRC");
 
@@ -1607,16 +1608,16 @@ NUMA     *nag;
  *      Return: 0 if OK; 1 on error
  *
  *  Notes:
- *      - in-place transform
- *      - see pixContrastTRC() and numaContrastTRC() in enhance.c for
- *        description and use of transform
+ *      (1) This is an in-place transform
+ *      (2) See pixContrastTRC() and numaContrastTRC() in enhance.c
+ *          for description and use of transform
  */
 l_int32
 pixcmapContrastTRC(PIXCMAP   *cmap,
                    l_float32  factor)
 {
-l_int32   i, ncolors, rval, gval, bval, trval, tgval, tbval;
-NUMA     *nac;
+l_int32  i, ncolors, rval, gval, bval, trval, tgval, tbval;
+NUMA    *nac;
 
     PROCNAME("pixcmapContrastTRC");
 
@@ -1652,20 +1653,20 @@ NUMA     *nac;
  *      Return: 0 if OK; 1 on error
  *
  *  Notes:
- *      - in-place transform
- *      - This does a proportional shift of the intensity for each color.
- *      - If fraction < 0.0, it moves all colors towards (0,0,0).
- *        This darkens the image.
- *      - If fraction > 0.0, it moves all colors towards (255,255,255)
- *        This fades the image.
- *      - The equivalent transform can be accomplished with pixcmapGammaTRC(),
- *        but it is considerably more difficult (see numaGammaTRC()).
+ *      (1) This is an in-place transform
+ *      (2) It does a proportional shift of the intensity for each color.
+ *      (3) If fraction < 0.0, it moves all colors towards (0,0,0).
+ *          This darkens the image.
+ *          If fraction > 0.0, it moves all colors towards (255,255,255)
+ *          This fades the image.
+ *      (4) The equivalent transform can be accomplished with pixcmapGammaTRC(),
+ *          but it is considerably more difficult (see numaGammaTRC()).
  */
 l_int32
 pixcmapShiftIntensity(PIXCMAP   *cmap,
                       l_float32  fraction)
 {
-l_int32   i, ncolors, rval, gval, bval;
+l_int32  i, ncolors, rval, gval, bval;
 
     PROCNAME("pixcmapShiftIntensity");
 
@@ -1692,3 +1693,43 @@ l_int32   i, ncolors, rval, gval, bval;
     return 0;
 }
 
+
+/*!
+ *  pixcmapShiftByComponent()
+ *
+ *      Input:  colormap
+ *              srcval (source color: 0xrrggbb00)
+ *              dstval (target color: 0xrrggbb00)
+ *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) This is an in-place transform
+ *      (2) It implements pixelShiftByComponent() for each color.
+ *          The mapping is specified by srcval and dstval.
+ *      (3) If a component decreases, the component in the colormap
+ *          decreases by the same ratio.  Likewise for increasing, except
+ *          all ratios are taken with respect to the distance from 255.
+ */
+l_int32
+pixcmapShiftByComponent(PIXCMAP  *cmap,
+                        l_uint32  srcval,
+                        l_uint32  dstval)
+{
+l_int32   i, ncolors, rval, gval, bval;
+l_uint32  newval;
+
+    PROCNAME("pixcmapShiftByComponent");
+
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+
+    ncolors = pixcmapGetCount(cmap);
+    for (i = 0; i < ncolors; i++) {
+        pixcmapGetColor(cmap, i, &rval, &gval, &bval);
+        pixelShiftByComponent(rval, gval, bval, srcval, dstval, &newval);
+        extractRGBValues(newval, &rval, &gval, &bval);
+        pixcmapResetColor(cmap, i, rval, gval, bval);
+    }
+
+    return 0;
+}
