@@ -40,7 +40,7 @@
  *           l_int32     pixGetPSNR()
  *
  *      Translated images at the same resolution
- *           TODO
+ *           l_int32     pixCompareWithTranslation()
  */
 
 #include <string.h>
@@ -494,13 +494,14 @@ PIX      *pixn;
  *           * unchanged:  black (on), white (off)
  *           * on in pix1, off in pix2: red
  *           * on in pix2, off in pix1: green
- *      (2) pix1 and pix2 must be the same size.
+ *      (2) This aligns the UL corners of pix1 and pix2, and crops
+ *          to the overlapping pixels.
  */
 PIX *
 pixDisplayDiffBinary(PIX  *pix1,
                      PIX  *pix2)
 {
-l_int32   w, h;
+l_int32   w1, h1, d1, w2, h2, d2, minw, minh;
 PIX      *pixt, *pixd;
 PIXCMAP  *cmap;
 
@@ -508,13 +509,14 @@ PIXCMAP  *cmap;
 
     if (!pix1 || !pix2)
         return (PIX *)ERROR_PTR("pix1, pix2 not both defined", procName, NULL);
-    if (!pixSizesEqual(pix1, pix2))
-        return (PIX *)ERROR_PTR("pix1 and pix2 unequal size", procName, NULL);
-    if (pixGetDepth(pix1) != 1)
+    pixGetDimensions(pix1, &w1, &h1, &d1);
+    pixGetDimensions(pix2, &w2, &h2, &d2);
+    if (d1 != 1 || d2 != 1)
         return (PIX *)ERROR_PTR("pix1 and pix2 not 1 bpp", procName, NULL);
+    minw = L_MIN(w1, w2);
+    minh = L_MIN(h1, h2);
 
-    pixGetDimensions(pix1, &w, &h, NULL);
-    pixd = pixCreate(w, h, 4);
+    pixd = pixCreate(minw, minh, 4);
     cmap = pixcmapCreate(4);
     pixcmapAddColor(cmap, 255, 255, 255);  /* initialized to white */
     pixcmapAddColor(cmap, 0, 0, 0);
@@ -1437,3 +1439,147 @@ l_float32  mse;  /* mean squared error */
     return 0;
 }
 
+
+/*------------------------------------------------------------------*
+ *             Translated images at the same resolution             *
+ *------------------------------------------------------------------*/
+/*!
+ *  pixCompareWithTranslation()
+ *
+ *      Input:  pix1, pix2 (any depth; colormap OK)
+ *              thresh (threshold for converting to 1 bpp)
+ *              &delx (<return> x translation on pix2 to align with pix1)
+ *              &dely (<return> y translation on pix2 to align with pix1)
+ *              &score (<return> correlation score at best alignment)
+ *              debugflag (1 for debug output; 0 for no debugging)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This does a coarse-to-fine search for best translational
+ *          alignment of two images, measured by a scoring function
+ *          that is the correlation between the fg pixels.
+ *      (2) The threshold is used if the images aren't 1 bpp.
+ *      (3) With debug on, you get a pdf that shows, as a grayscale
+ *          image, the score as a function of shift from the initial
+ *          estimate, for each of the four levels.  The shift is 0 at
+ *          the center of the image.
+ *      (4) With debug on, you also get a pdf that shows the
+ *          difference at the best alignment between the two images,
+ *          at each of the four levels.  The red and green pixels
+ *          show locations where one image has a fg pixel and the
+ *          other doesn't.  The black pixels are where both images
+ *          have fg pixels, and white pixels are where neither image
+ *          has fg pixels.
+ */
+l_int32
+pixCompareWithTranslation(PIX        *pix1,
+                          PIX        *pix2,
+                          l_int32     thresh,
+                          l_int32    *pdelx,
+                          l_int32    *pdely,
+                          l_float32  *pscore,
+                          l_int32     debugflag)
+{
+l_uint8   *subtab;
+l_int32    i, level, area1, area2, delx, dely;
+l_int32    etransx, etransy, maxshift, dbint;
+l_int32   *stab, *ctab;
+l_float32  cx1, cx2, cy1, cy2, score;
+PIX       *pixb1, *pixb2, *pixt1, *pixt2, *pixt3, *pixt4;
+PIXA      *pixa1, *pixa2, *pixadb;
+
+    PROCNAME("pixCompareWithTranslation");
+
+    if (!pdelx || !pdely)
+        return ERROR_INT("&delx and &dely not defined", procName, 1);
+    *pdelx = *pdely = 0;
+    if (!pscore)
+        return ERROR_INT("&score not defined", procName, 1);
+    *pscore = 0.0;
+    if (!pix1)
+        return ERROR_INT("pix1 not defined", procName, 1);
+    if (!pix2)
+        return ERROR_INT("pix2 not defined", procName, 1);
+
+        /* Make tables */
+    subtab = makeSubsampleTab2x();
+    stab = makePixelSumTab8();
+    ctab = makePixelCentroidTab8();
+
+        /* Binarize each image */
+    pixb1 = pixConvertTo1(pix1, thresh);
+    pixb2 = pixConvertTo1(pix2, thresh);
+
+        /* Make a cascade of 2x reduced images for each, thresholding
+         * with level 2 (neutral), down to 8x reduction */
+    pixa1 = pixaCreate(4);
+    pixa2 = pixaCreate(4);
+    if (debugflag)
+        pixadb = pixaCreate(4);
+    pixaAddPix(pixa1, pixb1, L_INSERT);
+    pixaAddPix(pixa2, pixb2, L_INSERT);
+    for (i = 0; i < 3; i++) {
+        pixt1 = pixReduceRankBinary2(pixb1, 2, subtab);
+        pixt2 = pixReduceRankBinary2(pixb2, 2, subtab);
+        pixaAddPix(pixa1, pixt1, L_INSERT);
+        pixaAddPix(pixa2, pixt2, L_INSERT);
+        pixb1 = pixt1;
+        pixb2 = pixt2;
+    }
+
+        /* At the lowest level, use the centroids with a maxshift of 6
+         * to search for the best alignment.  Then at higher levels,
+         * use the result from the level below as the initial approximation
+         * for the alignment, and search with a maxshift of 2. */
+    for (level = 3; level >= 0; level--) {
+        pixt1 = pixaGetPix(pixa1, level, L_CLONE);
+        pixt2 = pixaGetPix(pixa2, level, L_CLONE);
+        pixCountPixels(pixt1, &area1, stab);
+        pixCountPixels(pixt2, &area2, stab);
+        if (level == 3) {
+            pixCentroid(pixt1, ctab, stab, &cx1, &cy1);
+            pixCentroid(pixt2, ctab, stab, &cx2, &cy2);
+            etransx = lept_roundftoi(cx1 - cx2);
+            etransy = lept_roundftoi(cy1 - cy2);
+            maxshift = 6;
+        }
+        else {
+            etransx = 2 * delx;
+            etransy = 2 * dely;
+            maxshift = 2;
+        }
+        dbint = (debugflag) ? level + 1 : 0;
+        pixBestCorrelation(pixt1, pixt2, area1, area2, etransx, etransy,
+                           maxshift, stab, &delx, &dely, &score, dbint);
+        if (debugflag) {
+            fprintf(stderr, "Level %d: delx = %d, dely = %d, score = %7.4f\n",
+                    level, delx, dely, score);
+            pixRasteropIP(pixt2, delx, dely, L_BRING_IN_WHITE);
+            pixt3 = pixDisplayDiffBinary(pixt1, pixt2);
+            pixt4 = pixExpandReplicate(pixt3, 8 / (1 << (3 - level)));
+            pixaAddPix(pixadb, pixt4, L_INSERT);
+            pixDestroy(&pixt3);
+        }
+        pixDestroy(&pixt1);
+        pixDestroy(&pixt2);
+    }
+
+    if (debugflag) {
+        pixaConvertToPdf(pixadb, 300, 1.0, L_FLATE_ENCODE, 0, NULL,
+                         "/tmp/junkcmp.pdf");
+        convertFilesToPdf("/tmp", "junkcorrel_", 30, 1.0, L_FLATE_ENCODE,
+                          0, "Correlation scores at levels 1 through 5",
+                          "/tmp/junkcorrel.pdf");
+        pixaDestroy(&pixadb);
+    }
+
+    *pdelx = delx;
+    *pdely = dely;
+    *pscore = score;
+    pixaDestroy(&pixa1);
+    pixaDestroy(&pixa2);
+    FREE(subtab);
+    FREE(stab);
+    FREE(ctab);
+    return 0;
+}
