@@ -19,17 +19,25 @@
  *     These are functions for computing correlation between
  *     pairs of 1 bpp images.
  *
+ *     Optimized 2 pix correlators (for jbig2 clustering)
  *         l_float32   pixCorrelationScore()
  *         l_int32     pixCorrelationScoreThresholded()
+ *
+ *     Simple 2 pix correlators
  *         l_float32   pixCorrelationScoreSimple()
+ *         l_float32   pixCorrelationScoreShifted()
+ *
+ *     Shift to find the best correlation
+ *         l_int32     pixBestCorrelation()
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <math.h>
 #include "allheaders.h"
 
 
+/* -------------------------------------------------------------------- *
+ *           Optimized 2 pix correlators (for jbig2 clustering)         *
+ * -------------------------------------------------------------------- */
 /*!
  *  pixCorrelationScore()
  *
@@ -325,7 +333,8 @@ l_float32  score;
         }
     }
 
-    score = (l_float32)(count * count) / (l_float32)(area1 * area2);
+    score = (l_float32)count * (l_float32)count /
+             ((l_float32)area1 * (l_float32)area2);
 /*    fprintf(stderr, "score = %7.3f, count = %d, area1 = %d, area2 = %d\n",
              score, count, area1, area2); */
     return score;
@@ -656,7 +665,8 @@ l_int32    threshold;
         }
     }
 
-    score = (l_float32)(count * count) / (l_float32)(area1 * area2);
+    score = (l_float32)count * (l_float32)count /
+             ((l_float32)area1 * (l_float32)area2);
     if (score >= score_threshold) {
         fprintf(stderr, "count %d < threshold %d but score %g >= score_threshold %g\n",
                 count, threshold, score, score_threshold);
@@ -665,6 +675,9 @@ l_int32    threshold;
 }
 
 
+/* -------------------------------------------------------------------- *
+ *             Simple 2 pix correlators (for jbig2 clustering)          *
+ * -------------------------------------------------------------------- */
 /*!
  *  pixCorrelationScoreSimple()
  *
@@ -742,8 +755,203 @@ PIX       *pixt;
     pixCountPixels(pixt, &count, tab);
     pixDestroy(&pixt);
 
-    score = (l_float32)(count * count) / (l_float32)(area1 * area2);
+    score = (l_float32)count * (l_float32)count /
+             ((l_float32)area1 * (l_float32)area2);
 /*    fprintf(stderr, "score = %7.3f, count = %d, area1 = %d, area2 = %d\n",
              score, count, area1, area2); */
     return score;
 }
+
+
+/*!
+ *  pixCorrelationScoreShifted()
+ *
+ *      Input:  pix1   (1 bpp)
+ *              pix2   (1 bpp)
+ *              area1  (number of on pixels in pix1)
+ *              area2  (number of on pixels in pix2)
+ *              delx (x translation of pix2 relative to pix1)
+ *              dely (y translation of pix2 relative to pix1)
+ *              tab    (sum tab for byte)
+ *      Return: correlation score 
+ *
+ *  Notes:
+ *      (1) This finds the correlation between two 1 bpp images,
+ *          when pix2 is shifted by (delx, dely) with respect
+ *          to each other.
+ *      (2) This is implemented by starting with a copy of pix1 and
+ *          ANDing its pixels with those of a shifted pix2.
+ *      (3) Get the pixel counts for area1 and area2 using piCountPixels().
+ *      (4) A good estimate for a shift that would maximize the correlation
+ *          is to align the centroids (cx1, cy1; cx2, cy2), giving the
+ *          relative translations etransx and etransy:
+ *             etransx = cx1 - cx2
+ *             etransy = cy1 - cy2
+ *          Typically delx is chosen to be near etransx; ditto for dely.
+ *          This function is used in pixBestCorrelation(), where the
+ *          translations delx and dely are varied to find the best alignment.
+ *      (5) We do not check the sizes of pix1 and pix2, because they should
+ *          be comparable.
+ */
+l_float32
+pixCorrelationScoreShifted(PIX      *pix1,
+                           PIX      *pix2,
+                           l_int32   area1,
+                           l_int32   area2,
+                           l_int32   delx,
+                           l_int32   dely,
+                           l_int32  *tab)
+{
+l_int32    w1, h1, w2, h2, count;
+l_float32  score;
+PIX       *pixt;
+
+    PROCNAME("pixCorrelationScoreSimple");
+
+    if (!pix1 || pixGetDepth(pix1) != 1)
+        return (l_float32)ERROR_FLOAT("pix1 not 1 bpp", procName, 0.0);
+    if (!pix2 || pixGetDepth(pix2) != 1)
+        return (l_float32)ERROR_FLOAT("pix2 not 1 bpp", procName, 0.0);
+    if (!tab)
+        return (l_float32)ERROR_FLOAT("tab not defined", procName, 0.0);
+    if (!area1 || !area2)
+        return (l_float32)ERROR_FLOAT("areas must be > 0", procName, 0.0);
+
+    pixGetDimensions(pix1, &w1, &h1, NULL);
+    pixGetDimensions(pix2, &w2, &h2, NULL);
+
+        /*  To insure that pixels are ON only within the
+         *  intersection of pix1 and the shifted pix2:
+         *  (1) Start with pixt cleared and equal in size to pix1.
+         *  (2) Blit the shifted pix2 onto pixt.  Then all ON pixels
+         *      are within the intersection of pix1 and the shifted pix2.
+         *  (3) AND pix1 with pixt. */
+    pixt = pixCreateTemplate(pix1);
+    pixRasterop(pixt, delx, dely, w2, h2, PIX_SRC, pix2, 0, 0);
+    pixRasterop(pixt, 0, 0, w1, h1, PIX_SRC & PIX_DST, pix1, 0, 0);
+    pixCountPixels(pixt, &count, tab);
+    pixDestroy(&pixt);
+
+    score = (l_float32)count * (l_float32)count /
+             ((l_float32)area1 * (l_float32)area2);
+    return score;
+}
+
+
+/* -------------------------------------------------------------------- *
+ *                  Shift to find the best correlation                  *
+ * -------------------------------------------------------------------- */
+/*!
+ *  pixBestCorrelation()
+ *
+ *      Input:  pix1   (1 bpp)
+ *              pix2   (1 bpp)
+ *              area1  (number of on pixels in pix1)
+ *              area2  (number of on pixels in pix2)
+ *              etransx (estimated x translation of pix2 to align with pix1)
+ *              etransy (estimated y translation of pix2 to align with pix1)
+ *              maxshift  (max x and y shift of pix2, around the estimated
+ *                          alignment location, relative to pix1)
+ *              tab8 (<optional> sum tab for ON pixels in byte; can be NULL)
+ *              &delx (<return> best x shift of pix2 relative to pix1
+ *              &dely (<return> best y shift of pix2 relative to pix1
+ *              &score (<optional return> maximum score found; can be NULL)
+ *              debugflag (<= 0 to skip; positive to generate output.
+ *                         The integer is used to label the debug image.)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This finds the actual shift (delx, dely) between two 1 bpp
+ *          images that maximizes correlation score when pix2 is shifted
+ *          by this amount relative to pix1.  Also returns the max score.
+ *      (2) Get the pixel counts using pixCountPixels().
+ *          Get the centroids using pixCentroid() and wrap them into a pta.
+ *      (3) The centroid of pix2 is shifted with respect to the centroid
+ *          of pix1 by all values between -maxshiftx and maxshiftx,
+ *          and likewise for the y shifts.  Therefore, the number of
+ *          correlations computed is:
+ *               (2 * maxshiftx + 1) * (2 * maxshifty + 1)
+ *          Consequently, if pix1 and pix2 are large, you should do this
+ *          in a coarse-to-fine sequence.
+ */
+l_int32
+pixBestCorrelation(PIX        *pix1,
+                   PIX        *pix2,
+                   l_int32     area1,
+                   l_int32     area2,
+                   l_int32     etransx,
+                   l_int32     etransy,
+                   l_int32     maxshift,
+                   l_int32    *tab8,
+                   l_int32    *pdelx,
+                   l_int32    *pdely,
+                   l_float32  *pscore,
+                   l_int32     debugflag)
+{
+l_int32    shiftx, shifty, delx, dely;
+l_int32   *tab;
+l_float32  maxscore, score;
+FPIX      *fpix;
+PIX       *pixt1, *pixt2;
+
+    PROCNAME("pixBestCorrelation");
+
+    if (!pdelx || !pdely)
+        return ERROR_INT("&delx and &dely not defined", procName, 1);
+    *pdelx = *pdely = 0;
+    if (pscore) *pscore = 0.0;
+    if (!pix1 || pixGetDepth(pix1) != 1)
+        return ERROR_INT("pix1 not defined or not 1 bpp", procName, 1);
+    if (!pix2 || pixGetDepth(pix2) != 1)
+        return ERROR_INT("pix2 not defined or not 1 bpp", procName, 1);
+    if (!area1 || !area2)
+        return ERROR_INT("areas must be > 0", procName, 1);
+
+    if (debugflag > 0)
+        fpix = fpixCreate(2 * maxshift + 1, 2 * maxshift + 1);
+
+    if (!tab8)
+        tab = makePixelSumTab8();
+    else
+        tab = tab8;
+
+        /* Search over a set of {shiftx, shifty} for the max */
+    maxscore = 0;
+    delx = etransx;
+    dely = etransy;
+    for (shifty = -maxshift; shifty <= maxshift; shifty++) {
+        for (shiftx = -maxshift; shiftx <= maxshift; shiftx++) {
+            score = pixCorrelationScoreShifted(pix1, pix2, area1, area2,
+                                               etransx + shiftx,
+                                               etransy + shifty, tab);
+            if (debugflag > 0) {
+                fpixSetPixel(fpix, maxshift + shiftx, maxshift + shifty,
+                             1000.0 * score);
+/*                fprintf(stderr, "(sx, sy) = (%d, %d): score = %7.4f\n",
+                        shiftx, shifty, score); */
+            }
+            if (score > maxscore) {
+                maxscore = score;
+                delx = etransx + shiftx;
+                dely = etransy + shifty;
+            }
+        }
+    }
+                
+    if (debugflag > 0) {
+        char  buf[128];
+        pixt1 = fpixDisplayMaxDynamicRange(fpix);
+        pixt2 = pixExpandReplicate(pixt1, 20);
+        snprintf(buf, sizeof(buf), "/tmp/junkcorrel_%d.png", debugflag);
+        pixWrite(buf, pixt2, IFF_PNG);
+        pixDestroy(&pixt1);
+        pixDestroy(&pixt2);
+        fpixDestroy(&fpix);
+    }
+
+    *pdelx = delx;
+    *pdely = dely;
+    if (pscore) *pscore = maxscore;
+    if (!tab8) FREE(tab);
+    return 0;
+}
