@@ -1488,7 +1488,8 @@ FPIX  *fpix;
 FPIX *
 fpixReadStream(FILE  *fp)
 {
-l_int32     w, h, nbytes, version;
+char        buf[256];
+l_int32     w, h, nbytes, xres, yres, version;
 l_float32  *data;
 FPIX       *fpix;
 
@@ -1504,11 +1505,23 @@ FPIX       *fpix;
     if (fscanf(fp, "w = %d, h = %d, nbytes = %d\n", &w, &h, &nbytes) != 3)
         return (FPIX *)ERROR_PTR("read fail for data size", procName, NULL);
 
+        /* Use fgets() and sscanf(); not fscanf(), for the last
+         * bit of header data before the float data.  The reason is
+         * that fscanf throws away white space, and if the float data
+         * happens to begin with ascii character(s) that are white
+         * space, it will swallow them and all will be lost!  */
+    if (fgets(buf, sizeof(buf), fp) == NULL)
+        return (FPIX *)ERROR_PTR("fgets read fail", procName, NULL);
+    if (sscanf(buf, "xres = %d, yres = %d\n", &xres, &yres) != 2)
+        return (FPIX *)ERROR_PTR("read fail for xres, yres", procName, NULL);
+
     if ((fpix = fpixCreate(w, h)) == NULL)
         return (FPIX *)ERROR_PTR("fpix not made", procName, NULL);
+    fpixSetResolution(fpix, xres, yres);
     data = fpixGetData(fpix);
     if (fread(data, 1, nbytes, fp) != nbytes)
         return (FPIX *)ERROR_PTR("read error for nbytes", procName, NULL);
+    fgetc(fp);  /* ending nl */
 
         /* Convert to little-endian if necessary */
     fpixEndianByteSwap(fpix, fpix);
@@ -1557,7 +1570,7 @@ l_int32
 fpixWriteStream(FILE  *fp,
                 FPIX  *fpix)
 {
-l_int32     w, h, nbytes;
+l_int32     w, h, nbytes, xres, yres;
 l_float32  *data;
 FPIX       *fpixt;
 
@@ -1573,10 +1586,13 @@ FPIX       *fpixt;
 
     fpixGetDimensions(fpixt, &w, &h);
     data = fpixGetData(fpixt);
-    nbytes = 4 * h * fpixGetWpl(fpixt);
+    nbytes = w * h * sizeof(l_float32);
+    fpixGetResolution(fpixt, &xres, &yres); 
     fprintf(fp, "\nFPix Version %d\n", FPIX_VERSION_NUMBER);
     fprintf(fp, "w = %d, h = %d, nbytes = %d\n", w, h, nbytes);
+    fprintf(fp, "xres = %d, yres = %d\n", xres, yres);
     fwrite(data, 1, nbytes, fp);
+    fprintf(fp, "\n");
 
     fpixDestroy(&fpixt);
     return 0;
@@ -1684,7 +1700,8 @@ DPIX  *dpix;
 DPIX *
 dpixReadStream(FILE  *fp)
 {
-l_int32     w, h, nbytes, version;
+char        buf[256];
+l_int32     w, h, nbytes, version, xres, yres;
 l_float64  *data;
 DPIX       *dpix;
 
@@ -1700,11 +1717,23 @@ DPIX       *dpix;
     if (fscanf(fp, "w = %d, h = %d, nbytes = %d\n", &w, &h, &nbytes) != 3)
         return (DPIX *)ERROR_PTR("read fail for data size", procName, NULL);
 
+        /* Use fgets() and sscanf(); not fscanf(), for the last
+         * bit of header data before the float data.  The reason is
+         * that fscanf throws away white space, and if the float data
+         * happens to begin with ascii character(s) that are white
+         * space, it will swallow them and all will be lost!  */
+    if (fgets(buf, sizeof(buf), fp) == NULL)
+        return (DPIX *)ERROR_PTR("fgets read fail", procName, NULL);
+    if (sscanf(buf, "xres = %d, yres = %d\n", &xres, &yres) != 2)
+        return (DPIX *)ERROR_PTR("read fail for xres, yres", procName, NULL);
+
     if ((dpix = dpixCreate(w, h)) == NULL)
         return (DPIX *)ERROR_PTR("dpix not made", procName, NULL);
+    dpixSetResolution(dpix, xres, yres);
     data = dpixGetData(dpix);
     if (fread(data, 1, nbytes, fp) != nbytes)
         return (DPIX *)ERROR_PTR("read error for nbytes", procName, NULL);
+    fgetc(fp);  /* ending nl */
 
         /* Convert to little-endian if necessary */
     dpixEndianByteSwap(dpix, dpix);
@@ -1753,7 +1782,7 @@ l_int32
 dpixWriteStream(FILE  *fp,
                 DPIX  *dpix)
 {
-l_int32     w, h, nbytes;
+l_int32     w, h, nbytes, xres, yres;
 l_float64  *data;
 DPIX       *dpixt;
 
@@ -1768,11 +1797,14 @@ DPIX       *dpixt;
     dpixt = dpixEndianByteSwap(NULL, dpix);
 
     dpixGetDimensions(dpixt, &w, &h);
+    dpixGetResolution(dpixt, &xres, &yres);
     data = dpixGetData(dpixt);
-    nbytes = 8 * h * dpixGetWpl(dpixt);
+    nbytes = w * h * sizeof(l_float64);
     fprintf(fp, "\nDPix Version %d\n", DPIX_VERSION_NUMBER);
     fprintf(fp, "w = %d, h = %d, nbytes = %d\n", w, h, nbytes);
+    fprintf(fp, "xres = %d, yres = %d\n", xres, yres);
     fwrite(data, 1, nbytes, fp);
+    fprintf(fp, "\n");
 
     dpixDestroy(&dpixt);
     return 0;
@@ -1883,4 +1915,3 @@ l_float32  val;
      fprintf(fp, "\n");
      return 0;
 }
-
