@@ -63,6 +63,7 @@
  *      Contour rendering on grayscale images
  *
  *          PIX        *pixRenderContours()
+ *          PIX        *fpixAutoRenderContours()
  *          PIX        *fpixRenderContours()
  *
  *  The line rendering functions are relatively crude, but they
@@ -72,13 +73,11 @@
  *  render this onto a Pix.
  */
 
-#include <stdio.h>
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
 
 
-
 /*------------------------------------------------------------------*
  *        Pta generation for arbitrary shapes built with lines      *
  *------------------------------------------------------------------*/
@@ -765,7 +764,7 @@ l_int32  i, n, x, y, w, h, d, maxval;
 /*!
  *  pixRenderPtaArb()
  *
- *      Input:  pix
+ *      Input:  pix (any depth, cmapped ok)
  *              pta (arbitrary set of points)
  *              rval, gval, bval
  *      Return: 0 if OK, 1 on error
@@ -1067,7 +1066,7 @@ PTA  *pta;
 /*!
  *  pixRenderBoxArb()
  *
- *      Input:  pix
+ *      Input:  pix (any depth, cmapped ok)
  *              box
  *              width  (thickness of box lines)
  *              rval, gval, bval
@@ -1182,7 +1181,7 @@ PTA  *pta;
 
 
 /*!
- *  pixRenderBoxArb()
+ *  pixRenderHashBoxArb()
  *
  *      Input:  pix
  *              box
@@ -1699,18 +1698,58 @@ PIX       *pixd;
 }
 
 
+/*!
+ *  fpixAutoRenderContours()
+ *
+ *      Input:  pixs (8 or 16 bpp; no colormap)
+ *              ncontours (> 1, < 500, typ. about 50)
+ *      Return: pixd (8 bpp), or null on error
+ *
+ *  Notes:
+ *      (1) The increment is set to get approximately @ncontours.
+ *      (2) The proximity to the target value for contour display
+ *          is set at 0.1 * increment.
+ *      (3) Negative contours are rendered in red; 0 or positive
+ *          contours are rendered in black.
+ */
+PIX *
+fpixAutoRenderContours(FPIX    *fpix,
+                       l_int32  ncontours)
+{
+l_float32  minval, maxval, incr;
+PIX       *pixd;
+
+    PROCNAME("fpixAutoRenderContours");
+
+    if (!fpix)
+        return (PIX *)ERROR_PTR("fpix not defined", procName, NULL);
+    if (ncontours < 2 || ncontours > 500)
+        return (PIX *)ERROR_PTR("ncontours < 2 or > 500", procName, NULL);
+
+    fpixGetMin(fpix, &minval, NULL, NULL);
+    fpixGetMax(fpix, &maxval, NULL, NULL);
+    if (minval == maxval)
+        return (PIX *)ERROR_PTR("all values in fpix are equal", procName, NULL);
+    incr = (maxval - minval) / ((l_float32)ncontours - 1);
+    pixd = fpixRenderContours(fpix, incr, 0.1 * incr);
+}
+
+
 /*!
  *  fpixRenderContours()
  *
  *      Input:  fpixs
- *              startval (value of lowest contour
- *              incr  (increment to next contour; must be > 0.0)
+ *              incr  (increment between contours; must be > 0.0)
  *              proxim (required proximity to target value; typ. 0.1 * incr)
- *      Return: pixd (1 bpp), or null on error
+ *      Return: pixd (8 bpp), or null on error
+ *
+ *  Notes:
+ *      (1) Values are displayed when val/incr is close to an integer.
+ *      (2) Negative contours are rendered in red; 0 or positive
+ *          contours are rendered in black.
  */
 PIX *
 fpixRenderContours(FPIX      *fpixs,
-                   l_float32  startval,
                    l_float32  incr,
                    l_float32  proxim)
 {
@@ -1719,6 +1758,7 @@ l_float32   val, invincr, finter, diff;
 l_uint32   *datad, *lined;
 l_float32  *datas, *lines;
 PIX        *pixd;
+PIXCMAP    *cmap;
 
     PROCNAME("fpixRenderContours");
 
@@ -1728,8 +1768,13 @@ PIX        *pixd;
         return (PIX *)ERROR_PTR("incr <= 0.0", procName, NULL);
 
     fpixGetDimensions(fpixs, &w, &h);
-    if ((pixd = pixCreate(w, h, 1)) == NULL)
+    if ((pixd = pixCreate(w, h, 8)) == NULL)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+    cmap = pixcmapCreate(8);
+    pixSetColormap(pixd, cmap);
+    pixcmapAddColor(cmap, 255, 255, 255);  /* white */
+    pixcmapAddColor(cmap, 0, 0, 0);  /* black */
+    pixcmapAddColor(cmap, 255, 0, 0);  /* red */
 
     datas = fpixGetData(fpixs);
     wpls = fpixGetWpl(fpixs);
@@ -1741,16 +1786,16 @@ PIX        *pixd;
         lined = datad + i * wpld;
         for (j = 0; j < w; j++) {
             val = lines[j];
-            if (val < startval)
-                continue;
-            finter = L_ABS(invincr * (val - startval));
+            finter = invincr * val;
             diff = finter - floorf(finter);
-            if (diff <= proxim)
-                SET_DATA_BIT(lined, j); 
+            if (diff <= proxim) {
+                if (val < 0.0)
+                    SET_DATA_BYTE(lined, j, 2);
+                else
+                    SET_DATA_BYTE(lined, j, 1);
+            }
         }
     }
 
     return pixd;
 }
-
-
