@@ -51,6 +51,7 @@
  *          l_int32       pixCopyColormap()
  *          l_int32       pixSizesEqual()
  *          l_int32       pixTransferAllData()
+ *          l_int32       pixSwapAndDestroy()
  *
  *    Pix accessors
  *          l_int32       pixGetWidth()
@@ -97,7 +98,7 @@
  *
  *
  *  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
- *      Important notes on direct management of pix image data 
+ *      Important notes on direct management of pix image data
  *  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  *
  *  Custom allocator and deallocator
@@ -145,7 +146,12 @@
  *      But if the refcount of pixs is greater than 1, it just copies
  *      the data and decrements the ref count.
  *
- *  (3) Use pixExtractData() to extract the image data from the pix
+ *  (3) Use pixSwapAndDestroy(pixd, &pixs) to replace pixs by an
+ *      existing pixd.  This is similar to pixTransferAllData(), but
+ *      simpler, in that it never makes any copies and if pixs is
+ *      cloned, the other references are not changed by this operation.
+ *
+ *  (4) Use pixExtractData() to extract the image data from the pix
  *      without copying if possible.  This could be used, for example,
  *      to convert from a pix to some other data structure with minimal
  *      heap allocation.  After the data is extracated, the pixels can
@@ -195,7 +201,7 @@ static struct PixMemoryManager  pix_mem_manager = {
     &malloc,
     &free
 };
-    
+
 static void *
 pix_malloc(size_t  size)
 {
@@ -305,8 +311,10 @@ l_uint32  *data;
     if ((pixd = pixCreateHeader(width, height, depth)) == NULL)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
     wpl = pixGetWpl(pixd);
-    if ((data = (l_uint32 *)pix_malloc(4 * wpl * height)) == NULL)
+    if ((data = (l_uint32 *)pix_malloc(4 * wpl * height)) == NULL) {
+        pixDestroy(&pixd);
         return (PIX *)ERROR_PTR("pix_malloc fail for data", procName, NULL);
+    }
     pixSetData(pixd, data);
     pixSetPadBits(pixd, 0);
     return pixd;
@@ -782,7 +790,7 @@ PIX     *pixs;
         if (copytext)
             pixCopyText(pixd, pixs);
     }
-  
+
     pixCopyResolution(pixd, pixs);
     pixCopyDimensions(pixd, pixs);
     if (copyformat)
@@ -793,7 +801,60 @@ PIX     *pixs;
     pixDestroy(ppixs);
     return 0;
 }
-        
+
+
+/*!
+ *  pixSwapAndDestroy()
+ *
+ *      Input:  &pixd (<optional, return> input pixd can be null,
+ *                     and it must be different from pixs)
+ *              &pixs (will be nulled after the swap)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Simple operation to change the handle name safely.
+ *          After this operation, the original image in pixd has
+ *          been destroyed, pixd points to what was pixs, and
+ *          the input pixs ptr has been nulled.
+ *      (2) This works safely whether or not pixs and pixd are cloned.
+ *          If pixs is cloned, the other handles still point to
+ *          the original image, with the ref count reduced by 1.
+ *      (3) Usage example:
+ *            Pix *pix1 = pixRead("...");
+ *            Pix *pix2 = function(pix1, ...);
+ *            pixSwapAndDestroy(&pix1, &pix2);
+ *            pixDestroy(&pix1);  // holds what was in pix2
+ *          Example with clones ([] shows ref count of image generated
+ *                               by the function):
+ *            Pix *pixs = pixRead("...");
+ *            Pix *pix1 = pixClone(pixs);
+ *            Pix *pix2 = function(pix1, ...);   [1]
+ *            Pix *pix3 = pixClone(pix2);   [1] --> [2]
+ *            pixSwapAndDestroy(&pix1, &pix2);
+ *            pixDestroy(&pixs);  // still holds read image
+ *            pixDestroy(&pix1);  // holds what was in pix2  [2] --> [1]
+ *            pixDestroy(&pix3);  // holds what was in pix2  [1] --> [0]
+ */
+l_int32
+pixSwapAndDestroy(PIX  **ppixd,
+                  PIX  **ppixs)
+{
+    PROCNAME("pixSwapAndDestroy");
+
+    if (!ppixd)
+        return ERROR_INT("&pixd not defined", procName, 1);
+    if (!ppixs)
+        return ERROR_INT("&pixs not defined", procName, 1);
+    if (*ppixs == NULL)
+        return ERROR_INT("pixs not defined", procName, 1);
+    if (ppixs == ppixd)  /* no-op */
+        return ERROR_INT("&pixd == &pixs", procName, 1);
+
+    pixDestroy(ppixd);
+    *ppixd = pixClone(*ppixs);
+    pixDestroy(ppixs);
+    return 0;
+}
 
 
 /*--------------------------------------------------------------------*
