@@ -959,7 +959,7 @@ PIXCMAP   *cmap;
  *  pixBlendInRect()
  *
  *      Input:  pixs (32 bpp rgb)
- *              box (in which all pixels will be blended)
+ *              box (<optional> in which all pixels will be blended)
  *              val  (blend value; 0xrrggbb00)
  *              fract (fraction of color to be blended with each pixel in pixs)
  *      Return: 0 if OK; 1 on error
@@ -967,6 +967,7 @@ PIXCMAP   *cmap;
  *  Notes:
  *      (1) This is an in-place function.  It blends the input color @val
  *          with the pixels in pixs in the specified rectangle.
+ *          If no rectangle is specified, it blends over the entire image.
  */
 l_int32
 pixBlendInRect(PIX       *pixs,
@@ -983,14 +984,28 @@ l_uint32  *datas, *lines;
 
     if (!pixs || pixGetDepth(pixs) != 32)
         return ERROR_INT("pixs not defined or not 32 bpp", procName, 1);
-    if (!box)
-        return ERROR_INT("box not defined", procName, 1);
 
-    boxGetGeometry(box, &bx, &by, &bw, &bh);
+    extractRGBValues(val, &rval, &gval, &bval);
     pixGetDimensions(pixs, &w, &h, NULL);
     datas = pixGetData(pixs);
     wpls = pixGetWpl(pixs);
-    extractRGBValues(val, &rval, &gval, &bval);
+    if (!box) {
+        for (i = 0; i < h; i++) {   /* scan over box */
+            lines = datas +  i * wpls;
+            for (j = 0; j < w; j++) {
+                val32 = *(lines + j);
+                extractRGBValues(val32, &prval, &pgval, &pbval);
+                prval = (l_int32)((1. - fract) * prval + fract * rval);
+                pgval = (l_int32)((1. - fract) * pgval + fract * gval);
+                pbval = (l_int32)((1. - fract) * pbval + fract * bval);
+                composeRGBPixel(prval, pgval, pbval, &val32);
+                *(lines + j) = val32;
+            }
+        }
+        return 0;
+    }
+
+    boxGetGeometry(box, &bx, &by, &bw, &bh);
     for (i = 0; i < bh; i++) {   /* scan over box */
         if (by + i < 0 || by + i >= h) continue;
         lines = datas + (by + i) * wpls;
@@ -1005,7 +1020,6 @@ l_uint32  *datas, *lines;
             *(lines + bx + j) = val32;
         }
     }
-
     return 0;
 }
 
@@ -2662,5 +2676,3 @@ l_setAlphaMaskBorder(l_float32  val1,
     AlphaMaskBorderVals[0] = val1;
     AlphaMaskBorderVals[1] = val2;
 }
-
-
