@@ -44,6 +44,7 @@
  *
  *    Foreground pixel counting in 1 bpp images
  *           l_int32     pixZero()
+ *           l_int32     pixForegroundFraction()
  *           l_int32     pixCountPixels()
  *           NUMA       *pixaCountPixels()
  *           l_int32     pixCountPixelsInRow()
@@ -55,6 +56,9 @@
  *           l_int32    *makePixelSumTab8()
  *           l_int32    *makePixelCentroidTab8()
  *
+ *    Pixel counting in gray and colormapped images
+ *           l_int32     pixCountArbInRect()
+ *
  *    Sum of pixel values
  *           l_int32     pixSumPixelValues()
  *
@@ -898,6 +902,11 @@ PIX       *pixd;
  *          we issue a warning and return a copy of the input pix.
  *          If you really want to set every pixel to the same value,
  *          use pixSetAllArbitrary().
+ *      (7) This is useful for compressing an RGBA image where the part
+ *          of the image that is fully transparent is random; compression
+ *          is typically improved by setting that region to a constant.
+ *          For rendering as a 3 component RGB image over a uniform
+ *          background, use pixAlphaBlendUniform().
  */
 PIX *
 pixSetUnderTransparency(PIX      *pixs,
@@ -1340,6 +1349,34 @@ l_uint32  *data, *line;
 }
 
 
+/*!
+ *  pixForegroundFraction()
+ *
+ *      Input:  pix (1 bpp)
+ *              &fract (<return> fraction of ON pixels)
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+pixForegroundFraction(PIX        *pix,
+                      l_float32  *pfract)
+{
+l_int32  w, h, count;
+
+    PROCNAME("pixForegroundFraction");
+
+    if (!pfract)
+        return ERROR_INT("pfract not defined", procName, 1);
+    *pfract = 0.0;
+    if (!pix || pixGetDepth(pix) != 1)
+        return ERROR_INT("pix not defined or not 1 bpp", procName, 1);
+
+    pixCountPixels(pix, &count, NULL);
+    pixGetDimensions(pix, &w, &h, NULL);
+    *pfract = (l_float32)count / (l_float32)(w * h);
+    return 0;
+}
+
+
 /*!
  *  pixCountPixels()
  *
@@ -1886,6 +1923,81 @@ l_int32  *tab;
 }
 
 
+/*-------------------------------------------------------------*
+ *        Pixel counting in gray and colormapped images        *
+ *-------------------------------------------------------------*/
+/*!
+ *  pixCountArbInRect()
+ *
+ *      Input:  pixs (8 bpp, or colormapped)
+ *              box (<optional>) over which count is made;
+ *                   use entire image null)
+ *              val (pixel value to count)
+ *              factor (subsampling factor; integer >= 1)
+ *              &count (<return> count; estimate it if factor > 1)
+ *      Return: na (histogram), or null on error
+ *
+ *  Notes:
+ *      (1) If pixs is cmapped, @val is compared to the colormap index;
+ *          otherwise, @val is compared to the grayscale value.
+ *      (2) Set the subsampling @factor > 1 to reduce the amount of computation.
+ *          If @factor > 1, multiply the count by @factor * @factor.
+ */
+l_int32
+pixCountArbInRect(PIX      *pixs,
+                  BOX      *box,
+                  l_int32   val,
+                  l_int32   factor,
+                  l_int32  *pcount)
+{
+l_int32    i, j, bx, by, bw, bh, w, h, wpl, pixval;
+l_uint32  *data, *line;
+
+    PROCNAME("pixCountArbInRect");
+
+    if (!pcount)
+        return ERROR_INT("&count not defined", procName, 1);
+    *pcount = 0;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+    if (pixGetDepth(pixs) != 8 && !pixGetColormap(pixs))
+        return ERROR_INT("pixs neither 8 bpp nor colormapped",
+                                 procName, 1);
+    if (factor < 1)
+        return ERROR_INT("sampling factor < 1", procName, 1);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    data = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
+
+    if (!box) {
+        for (i = 0; i < h; i += factor) {
+            line = data + i * wpl;
+            for (j = 0; j < w; j += factor) {
+                pixval = GET_DATA_BYTE(line, j);
+                if (pixval == val) (*pcount)++;
+            }
+        }
+    }
+    else {
+        boxGetGeometry(box, &bx, &by, &bw, &bh);
+        for (i = 0; i < bh; i += factor) {
+            if (by + i < 0 || by + i >= h) continue;
+            line = data + (by + i) * wpl;
+            for (j = 0; j < bw; j += factor) {
+                if (bx + j < 0 || bx + j >= w) continue;
+                pixval = GET_DATA_BYTE(line, bx + j);
+                if (pixval == val) (*pcount)++;
+            }
+        }
+    }
+
+    if (factor > 1)  /* assume pixel color is randomly distributed */
+        *pcount = *pcount * factor * factor;
+    return 0;
+}
+
+
 /*-------------------------------------------------------------*
  *                       Sum of pixel values                   *
  *-------------------------------------------------------------*/
@@ -2162,4 +2274,3 @@ l_uint32  val, maxval;
     pixGetPixel(pixs, *pxc, *pyc, pdist);
     return 0;
 }
-
