@@ -65,6 +65,7 @@
  *           l_int32   pixaaGetCount()
  *           PIXA     *pixaaGetPixa()
  *           BOXA     *pixaaGetBoxa()
+ *           PIX      *pixaaGetPix()
  *
  *      Pixa serialized I/O  (requires png support)
  *           PIXA     *pixaRead()
@@ -1440,6 +1441,35 @@ pixaaGetBoxa(PIXAA   *pixaa,
 }
 
 
+/*!
+ *  pixaaGetPix()
+ *
+ *      Input:  paa
+ *              ipixa  (index into the pixa array in the pixaa)
+ *              ipix  (index into the pix array in the pixa)
+ *              accessflag  (L_COPY or L_CLONE)
+ *      Return: pix, or null on error
+ */
+PIX *
+pixaaGetPix(PIXAA   *paa,
+            l_int32  ipixa,
+            l_int32  ipix,
+            l_int32  accessflag)
+{
+PIX   *pix;
+PIXA  *pixa;
+
+    PROCNAME("pixaaGetPix");
+
+    if ((pixa = pixaaGetPixa(paa, ipixa, L_CLONE)) == NULL)
+        return (PIX *)ERROR_PTR("pixa not retrieved", procName, NULL);
+    if ((pix = pixaGetPix(pixa, ipix, accessflag)) == NULL)
+        L_ERROR("pix not retrieved", procName);
+    pixaDestroy(&pixa);
+    return pix;
+}
+
+
 /*---------------------------------------------------------------------*
  *                          Pixa serialized I/O                        *
  *---------------------------------------------------------------------*/
@@ -1800,5 +1830,3 @@ PIXA    *pixa;
     }
     return 0;
 }
-
-
