@@ -54,6 +54,8 @@
  *           l_int32   pixacompGetBoxaCount()
  *           BOX      *pixacompGetBox()
  *           l_int32   pixacompGetBoxGeometry()
+ *           l_int32   pixacompGetOffset()
+ *           l_int32   pixacompSetOffset()
  *
  *      Pixacomp conversion to Pixa
  *           PIXA     *pixaCreateFromPixacomp()
@@ -64,6 +66,10 @@
  *           l_int32   pixacompWrite()
  *           l_int32   pixacompWriteStream()
  *
+ *      Conversion to pdf
+ *           l_int32   pixacompConvertToPdf()
+ *           l_int32   pixacompConvertToPdfData()
+ *
  *      Output for debugging
  *           l_int32   pixacompWriteStreamInfo()
  *           l_int32   pixcompWriteStreamInfo()
@@ -87,10 +93,15 @@
  *   serialization of the Pixacomp does not require any imaging
  *   libraries because it simply reads and writes the compressed data.
  *
- *   For random insertion and replacement of pixcomp into a pixcomp,
+ *   For random insertion and replacement of pixcomp into a pixacomp,
  *   initialize a fully populated array using pixacompCreateInitialized().
  *   Then use pixacompReplacePix() or pixacompReplacePixcomp() for
  *   the random insertion.
+ *
+ *   The offset field allows you to use an offset-based index to
+ *   access the 0-based ptr array in the pixacomp.  This would typically
+ *   be used to map the pixacomp array index to a page number, or v.v.
+ *   By default, the offset is 0.
  */
 
 #include <string.h>
@@ -248,7 +259,7 @@ PIXC     *pixc;
     if ((format == IFF_TIFF_G4 && comptype != IFF_PNG) ||
         (format == IFF_JFIF_JPEG && comptype != IFF_PNG))
         comptype = format;
-    if (comptype != IFF_DEFAULT && comptype == format) { 
+    if (comptype != IFF_DEFAULT && comptype == format) {
         data = l_binaryRead(filename, &nbytes);
         if ((pixc = pixcompCreateFromString(data, nbytes, L_INSERT)) == NULL) {
             FREE(data);
@@ -428,10 +439,9 @@ PIX     *pix;
         L_ERROR_INT("pix comptype %d not equal to pixc comptype",
                     procName, format);
     }
-    
+
     return pix;
 }
- 
 
 
 /*---------------------------------------------------------------------*
@@ -457,7 +467,8 @@ PIXAC  *pixac;
         return (PIXAC *)ERROR_PTR("pixac not made", procName, NULL);
     pixac->n = 0;
     pixac->nalloc = n;
-    
+    pixac->offset = 0;
+
     if ((pixac->pixc = (PIXC **)CALLOC(n, sizeof(PIXC *))) == NULL)
         return (PIXAC *)ERROR_PTR("pixc ptrs not made", procName, NULL);
     if ((pixac->boxa = boxaCreate(n)) == NULL)
@@ -471,6 +482,7 @@ PIXAC  *pixac;
  *  pixacompCreateInitialized()
  *
  *      Input:  n  (initial number of ptrs)
+ *              offset (difference: accessor index - pixacomp array index)
  *              pix (initialize each ptr in pixacomp to this pix)
  *              comptype (IFF_DEFAULT, IFF_TIFF_G4, IFF_PNG, IFF_JFIF_JPEG)
  *      Return: pixac, or null on error
@@ -480,19 +492,23 @@ PIXAC  *pixac;
  *      (2) Typically use a very small @pix (w = h = 1) with
  *          @comptype == IFF_TIFF_G4 for the initialization.
  *      (3) Example usage:
+ *            // Generate pixacomp for pages 30 - 49.  This has an array
+ *            // size of 20 and the page number offset is 30.
  *            Pix *pix = pixCreate(1, 1, 1);
- *            Pixacomp *pixac = pixacompCreateInitialized(50, pix, IFF_TIFF_G4);
- *            for (i = 0; i < 50; i++) {
- *                Pix *pixt = ...
+ *            Pixacomp *pixac = pixacompCreateInitialized(20, 30, pix,
+ *                                                        IFF_TIFF_G4);
+ *            for (pageno = 30; pageno < 50; pageno++) {
+ *                Pix *pixt = ...   // derived from image[pageno]
  *                if (pixt)
- *                    pixacompReplacePix(pixac, i, pixt, IFF_TIFF_G4);
+ *                    pixacompReplacePix(pixac, pageno, pixt, IFF_TIFF_G4);
  *                pixDestroy(&pixt);
  *            }
- *          The result is a fully populated pixac with selected pixt
- *          replacing the placeholders.
+ *          The result is a pixac with 20 compressed strings, and with
+ *          selected pixt replacing the placeholders.
  */
 PIXAC *
 pixacompCreateInitialized(l_int32  n,
+                          l_int32  offset,
                           PIX     *pix,
                           l_int32  comptype)
 {
@@ -504,11 +520,16 @@ PIXAC   *pixac;
 
     if (n <= 0)
         return (PIXAC *)ERROR_PTR("n must be > 0", procName, NULL);
+    if (offset < 0) {
+        L_WARNING("offset < 0; setting to 0", procName);
+        offset = 0;
+    }
     if (!pix)
         return (PIXAC *)ERROR_PTR("pix not defined", procName, NULL);
 
     if ((pixac = pixacompCreate(n)) == NULL)
         return (PIXAC *)ERROR_PTR("pixac not made", procName, NULL);
+    pixacompSetOffset(pixac, offset);
     for (i = 0; i < n; i++) {
         pixc = pixcompCreateFromPix(pix, comptype);
         pixacompAddPixcomp(pixac, pixc);
@@ -706,6 +727,11 @@ PIXAC   *pixac;
  *              pix  (to be added)
  *              comptype (IFF_DEFAULT, IFF_TIFF_G4, IFF_PNG, IFF_JFIF_JPEG)
  *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) The array is filled up to the (n-1)-th element, and this
+ *          converts the input pix to a pixcomp and adds it at
+ *          the n-th position.
  */
 l_int32
 pixacompAddPix(PIXAC   *pixac,
@@ -798,13 +824,15 @@ pixacompExtendArray(PIXAC  *pixac)
  *  pixacompReplacePix()
  *
  *      Input:  pixac
- *              index (of pixc within pixac to be replaced)
+ *              index (caller's view of index within pixac; includes offset)
  *              pix  (owned by the caller)
  *              comptype (IFF_DEFAULT, IFF_TIFF_G4, IFF_PNG, IFF_JFIF_JPEG)
  *      Return: 0 if OK; 1 on error
  *
  *  Notes:
- *      (1) The input @pix is converted to a pixc, which is then inserted
+ *      (1) The @index includes the offset, which must be subtracted
+ *          to get the actual index into the ptr array.
+ *      (2) The input @pix is converted to a pixc, which is then inserted
  *          into the pixac.
  */
 l_int32
@@ -813,7 +841,7 @@ pixacompReplacePix(PIXAC   *pixac,
                    PIX     *pix,
                    l_int32  comptype)
 {
-l_int32  n;
+l_int32  n, aindex;
 PIXC    *pixc;
 
     PROCNAME("pixacompReplacePix");
@@ -821,7 +849,8 @@ PIXC    *pixc;
     if (!pixac)
         return ERROR_INT("pixac not defined", procName, 1);
     n = pixacompGetCount(pixac);
-    if (index < 0 || index >= n)
+    aindex = index - pixac->offset;
+    if (aindex < 0 || aindex >= n)
         return ERROR_INT("array index out of bounds", procName, 1);
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
@@ -839,12 +868,14 @@ PIXC    *pixc;
  *  pixacompReplacePixcomp()
  *
  *      Input:  pixac
- *              index (of pixc within pixac to be replaced)
+ *              index (caller's view of index within pixac; includes offset)
  *              pixc  (to replace existing one, which is destroyed)
  *      Return: 0 if OK; 1 on error
  *
  *  Notes:
- *      (1) The inserted @pixc is now owned by the pixac.  The caller
+ *      (1) The @index includes the offset, which must be subtracted
+ *          to get the actual index into the ptr array.
+ *      (2) The inserted @pixc is now owned by the pixac.  The caller
  *          must not destroy it.
  */
 l_int32
@@ -852,7 +883,7 @@ pixacompReplacePixcomp(PIXAC   *pixac,
                        l_int32  index,
                        PIXC    *pixc)
 {
-l_int32  n;
+l_int32  n, aindex;
 PIXC    *pixct;
 
     PROCNAME("pixacompReplacePixcomp");
@@ -860,14 +891,15 @@ PIXC    *pixct;
     if (!pixac)
         return ERROR_INT("pixac not defined", procName, 1);
     n = pixacompGetCount(pixac);
-    if (index < 0 || index >= n)
+    aindex = index - pixac->offset;
+    if (aindex < 0 || aindex >= n)
         return ERROR_INT("array index out of bounds", procName, 1);
     if (!pixc)
         return ERROR_INT("pixc not defined", procName, 1);
 
-    pixct = pixacompGetPixcomp(pixac, index);
+    pixct = pixacompGetPixcomp(pixac, index);  /* use @index */
     pixcompDestroy(&pixct);
-    pixac->pixc[index] = pixc;  /* replace */
+    pixac->pixc[aindex] = pixc;  /* replace; use array index */
 
     return 0;
 }
@@ -925,25 +957,30 @@ pixacompGetCount(PIXAC  *pixac)
  *  pixacompGetPixcomp()
  *
  *      Input:  pixac
- *              index  (to the index-th pix)
+ *              index (caller's view of index within pixac; includes offset)
  *      Return: pixc, or null on error
  *
  *  Notes:
- *      (1) Important: this is just a ptr to the pixc owned by the pixac.
+ *      (1) The @index includes the offset, which must be subtracted
+ *          to get the actual index into the ptr array.
+ *      (2) Important: this is just a ptr to the pixc owned by the pixac.
  *          Do not destroy unless you are replacing the pixc.
  */
 PIXC *
 pixacompGetPixcomp(PIXAC   *pixac,
                    l_int32  index)
 {
+l_int32  aindex;
+
     PROCNAME("pixacompGetPixcomp");
 
     if (!pixac)
         return (PIXC *)ERROR_PTR("pixac not defined", procName, NULL);
-    if (index < 0 || index >= pixac->n)
-        return (PIXC *)ERROR_PTR("index not valid", procName, NULL);
+    aindex = index - pixac->offset;
+    if (aindex < 0 || aindex >= pixac->n)
+        return (PIXC *)ERROR_PTR("array index not valid", procName, NULL);
 
-    return pixac->pixc[index];
+    return pixac->pixc[aindex];
 }
 
 
@@ -951,21 +988,27 @@ pixacompGetPixcomp(PIXAC   *pixac,
  *  pixacompGetPix()
  *
  *      Input:  pixac
- *              index  (to the index-th pix)
+ *              index (caller's view of index within pixac; includes offset)
  *      Return: pix, or null on error
+ *
+ *  Notes:
+ *      (1) The @index includes the offset, which must be subtracted
+ *          to get the actual index into the ptr array.
  */
 PIX *
 pixacompGetPix(PIXAC   *pixac,
                l_int32  index)
 {
-PIXC  *pixc;
+l_int32  aindex;
+PIXC    *pixc;
 
     PROCNAME("pixacompGetPix");
 
     if (!pixac)
         return (PIX *)ERROR_PTR("pixac not defined", procName, NULL);
-    if (index < 0 || index >= pixac->n)
-        return (PIX *)ERROR_PTR("index not valid", procName, NULL);
+    aindex = index - pixac->offset;
+    if (aindex < 0 || aindex >= pixac->n)
+        return (PIX *)ERROR_PTR("array index not valid", procName, NULL);
 
     pixc = pixacompGetPixcomp(pixac, index);
     return pixCreateFromPixcomp(pixc);
@@ -976,9 +1019,13 @@ PIXC  *pixc;
  *  pixacompGetPixDimensions()
  *
  *      Input:  pixa
- *              index  (to the index-th box)
+ *              index (caller's view of index within pixac; includes offset)
  *              &w, &h, &d (<optional return>; each can be null)
  *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) The @index includes the offset, which must be subtracted
+ *          to get the actual index into the ptr array.
  */
 l_int32
 pixacompGetPixDimensions(PIXAC    *pixac,
@@ -987,16 +1034,18 @@ pixacompGetPixDimensions(PIXAC    *pixac,
                          l_int32  *ph,
                          l_int32  *pd)
 {
-PIXC  *pixc;
+l_int32  aindex;
+PIXC    *pixc;
 
     PROCNAME("pixacompGetPixDimensions");
 
     if (!pixac)
         return ERROR_INT("pixac not defined", procName, 1);
-    if (index < 0 || index >= pixac->n)
-        return ERROR_INT("index not valid", procName, 1);
+    aindex = index - pixac->offset;
+    if (aindex < 0 || aindex >= pixac->n)
+        return ERROR_INT("array index not valid", procName, 1);
 
-    if ((pixc = pixac->pixc[index]) == NULL)
+    if ((pixc = pixac->pixc[aindex]) == NULL)
         return ERROR_INT("pixc not found!", procName, 1);
     pixcompGetDimensions(pixc, pw, ph, pd);
     return 0;
@@ -1041,7 +1090,7 @@ pixacompGetBoxaCount(PIXAC  *pixac)
 
     if (!pixac)
         return ERROR_INT("pixac not defined", procName, 0);
-    
+
     return boxaGetCount(pixac->boxa);
 }
 
@@ -1050,16 +1099,18 @@ pixacompGetBoxaCount(PIXAC  *pixac)
  *  pixacompGetBox()
  *
  *      Input:  pixac
- *              index  (to the index-th pix)
+ *              index (caller's view of index within pixac; includes offset)
  *              accesstype  (L_COPY or L_CLONE)
  *      Return: box (if null, not automatically an error), or null on error
  *
  *  Notes:
- *      (1) There is always a boxa with a pixac, and it is initialized so
+ *      (1) The @index includes the offset, which must be subtracted
+ *          to get the actual index into the ptr array.
+ *      (2) There is always a boxa with a pixac, and it is initialized so
  *          that each box ptr is NULL.
- *      (2) In general, we expect that there is either a box associated
+ *      (3) In general, we expect that there is either a box associated
  *          with each pixc, or no boxes at all in the boxa.
- *      (3) Having no boxes is thus not an automatic error.  Whether it
+ *      (4) Having no boxes is thus not an automatic error.  Whether it
  *          is an actual error is determined by the calling program.
  *          If the caller expects to get a box, it is an error; see, e.g.,
  *          pixacGetBoxGeometry().
@@ -1069,7 +1120,8 @@ pixacompGetBox(PIXAC    *pixac,
                l_int32   index,
                l_int32   accesstype)
 {
-BOX  *box;
+l_int32  aindex;
+BOX     *box;
 
     PROCNAME("pixacompGetBox");
 
@@ -1077,12 +1129,13 @@ BOX  *box;
         return (BOX *)ERROR_PTR("pixac not defined", procName, NULL);
     if (!pixac->boxa)
         return (BOX *)ERROR_PTR("boxa not defined", procName, NULL);
-    if (index < 0 || index >= pixac->boxa->n)
-        return (BOX *)ERROR_PTR("index not valid", procName, NULL);
+    aindex = index - pixac->offset;
+    if (aindex < 0 || aindex >= pixac->boxa->n)
+        return (BOX *)ERROR_PTR("array index not valid", procName, NULL);
     if (accesstype != L_COPY && accesstype != L_CLONE)
         return (BOX *)ERROR_PTR("invalid accesstype", procName, NULL);
 
-    box = pixac->boxa->box[index];
+    box = pixac->boxa->box[aindex];
     if (box) {
         if (accesstype == L_COPY)
             return boxCopy(box);
@@ -1098,28 +1151,34 @@ BOX  *box;
  *  pixacompGetBoxGeometry()
  *
  *      Input:  pixac
- *              index  (to the index-th box)
+ *              index (caller's view of index within pixac; includes offset)
  *              &x, &y, &w, &h (<optional return>; each can be null)
  *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) The @index includes the offset, which must be subtracted
+ *          to get the actual index into the ptr array.
  */
 l_int32
-pixacompGetBoxGeometry(PIXAC     *pixac,
-                       l_int32    index,
-                       l_int32   *px,
-                       l_int32   *py,
-                       l_int32   *pw,
+pixacompGetBoxGeometry(PIXAC    *pixac,
+                       l_int32   index,
+                       l_int32  *px,
+                       l_int32  *py,
+                       l_int32  *pw,
                        l_int32  *ph)
 {
-BOX  *box;
+l_int32  aindex;
+BOX     *box;
 
     PROCNAME("pixacompGetBoxGeometry");
 
     if (!pixac)
         return ERROR_INT("pixac not defined", procName, 1);
-    if (index < 0 || index >= pixac->n)
-        return ERROR_INT("index not valid", procName, 1);
+    aindex = index - pixac->offset;
+    if (aindex < 0 || aindex >= pixac->n)
+        return ERROR_INT("array index not valid", procName, 1);
 
-    if ((box = pixacompGetBox(pixac, index, L_CLONE)) == NULL)
+    if ((box = pixacompGetBox(pixac, aindex, L_CLONE)) == NULL)
         return ERROR_INT("box not found!", procName, 1);
     boxGetGeometry(box, px, py, pw, ph);
     boxDestroy(&box);
@@ -1127,6 +1186,53 @@ BOX  *box;
 }
 
 
+/*!
+ *  pixacompGetOffset()
+ *
+ *      Input:  pixac
+ *      Return: offset, or 0 on error
+ *
+ *  Notes:
+ *      (1) The offset is the difference between the caller's view of
+ *          the index into the array and the actual array index.
+ *          By default it is 0.
+ */
+l_int32
+pixacompGetOffset(PIXAC   *pixac)
+{
+    PROCNAME("pixacompGetOffset");
+
+    if (!pixac)
+        return ERROR_INT("pixac not defined", procName, 0);
+    return pixac->offset;
+}
+
+
+/*!
+ *  pixacompSetOffset()
+ *
+ *      Input:  pixac
+ *              offset (non-negative)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) The offset is the difference between the caller's view of
+ *          the index into the array and the actual array index.
+ *          By default it is 0.
+ */
+l_int32
+pixacompSetOffset(PIXAC   *pixac,
+                  l_int32  offset)
+{
+    PROCNAME("pixacompSetOffset");
+
+    if (!pixac)
+        return ERROR_INT("pixac not defined", procName, 1);
+    pixac->offset = L_MAX(0, offset);
+    return 0;
+}
+
+
 /*---------------------------------------------------------------------*
  *                      Pixacomp conversion to Pixa                    *
  *---------------------------------------------------------------------*/
@@ -1218,8 +1324,9 @@ PIXAC  *pixac;
 PIXAC *
 pixacompReadStream(FILE  *fp)
 {
+char      buf[256];
 l_uint8  *data;
-l_int32   n, i, w, h, d, ignore;
+l_int32   n, offset, i, w, h, d, ignore;
 l_int32   comptype, size, cmapflag, version, xres, yres;
 BOXA     *boxa;
 PIXC     *pixc;
@@ -1236,6 +1343,8 @@ PIXAC    *pixac;
         return (PIXAC *)ERROR_PTR("invalid pixacomp version", procName, NULL);
     if (fscanf(fp, "Number of pixcomp = %d", &n) != 1)
         return (PIXAC *)ERROR_PTR("not a pixacomp file", procName, NULL);
+    if (fscanf(fp, "Offset of index into array = %d", &offset) != 1)
+        return (PIXAC *)ERROR_PTR("offset not read", procName, NULL);
 
     if ((pixac = pixacompCreate(n)) == NULL)
         return (PIXAC *)ERROR_PTR("pixac not made", procName, NULL);
@@ -1243,22 +1352,33 @@ PIXAC    *pixac;
         return (PIXAC *)ERROR_PTR("boxa not made", procName, NULL);
     boxaDestroy(&pixac->boxa);  /* empty */
     pixac->boxa = boxa;
+    pixacompSetOffset(pixac, offset);
 
     for (i = 0; i < n; i++) {
         if ((pixc = (PIXC *)CALLOC(1, sizeof(PIXC))) == NULL)
             return (PIXAC *)ERROR_PTR("pixc not made", procName, NULL);
-        if (fscanf(fp, "  Pixcomp[%d]: w = %d, h = %d, d = %d\n",
+        if (fscanf(fp, "\nPixcomp[%d]: w = %d, h = %d, d = %d\n",
                    &ignore, &w, &h, &d) != 4)
             return (PIXAC *)ERROR_PTR("size reading", procName, NULL);
-        if (fscanf(fp, "    comptype = %d, size = %d, cmapflag = %d\n",
+        if (fscanf(fp, "  comptype = %d, size = %d, cmapflag = %d\n",
                    &comptype, &size, &cmapflag) != 3)
             return (PIXAC *)ERROR_PTR("comptype/size reading", procName, NULL);
-        if (fscanf(fp, "    xres = %d, yres = %d\n", &xres, &yres) != 2)
-            return (PIXAC *)ERROR_PTR("res reading", procName, NULL);
+
+           /* Use fgets() and sscanf(); not fscanf(), for the last
+             * bit of header data before the binary data.  The reason is
+             * that fscanf throws away white space, and if the binary data
+             * happens to begin with ascii character(s) that are white
+             * space, it will swallow them and all will be lost!  */
+        if (fgets(buf, sizeof(buf), fp) == NULL)
+            return (PIXAC *)ERROR_PTR("fgets read fail", procName, NULL);
+        if (sscanf(buf, "  xres = %d, yres = %d\n", &xres, &yres) != 2)
+            return (PIXAC *)ERROR_PTR("read fail for res", procName, NULL);
+
         if ((data = (l_uint8 *)CALLOC(1, size)) == NULL)
             return (PIXAC *)ERROR_PTR("calloc fail for data", procName, NULL);
         if (fread(data, 1, size, fp) != size)
             return (PIXAC *)ERROR_PTR("error reading data", procName, NULL);
+        fgetc(fp);  /* swallow the ending nl */
         pixc->w = w;
         pixc->h = h;
         pixc->d = d;
@@ -1319,7 +1439,7 @@ l_int32
 pixacompWriteStream(FILE   *fp,
                     PIXAC  *pixac)
 {
-l_int32  n, i;
+l_int32  n, offset, i;
 PIXC    *pixc;
 
     PROCNAME("pixacompWriteStream");
@@ -1332,21 +1452,195 @@ PIXC    *pixc;
     n = pixacompGetCount(pixac);
     fprintf(fp, "\nPixacomp Version %d\n", PIXACOMP_VERSION_NUMBER);
     fprintf(fp, "Number of pixcomp = %d", n);
+    fprintf(fp, "Offset of index into array = %d", offset);
     boxaWriteStream(fp, pixac->boxa);
     for (i = 0; i < n; i++) {
-        if ((pixc = pixacompGetPixcomp(pixac, i)) == NULL)
+        if ((pixc =
+             pixacompGetPixcomp(pixac, pixacompGetOffset(pixac) + i)) == NULL)
             return ERROR_INT("pixc not found", procName, 1);
-        fprintf(fp, "  Pixcomp[%d]: w = %d, h = %d, d = %d\n",
+        fprintf(fp, "\nPixcomp[%d]: w = %d, h = %d, d = %d\n",
                 i, pixc->w, pixc->h, pixc->d);
-        fprintf(fp, "    comptype = %d, size = %ld, cmapflag = %d\n",
+        fprintf(fp, "  comptype = %d, size = %ld, cmapflag = %d\n",
                 pixc->comptype, pixc->size, pixc->cmapflag);
-        fprintf(fp, "    xres = %d, yres = %d\n", pixc->xres, pixc->yres);
+        fprintf(fp, "  xres = %d, yres = %d\n", pixc->xres, pixc->yres);
         fwrite(pixc->data, 1, pixc->size, fp);
+        fprintf(fp, "\n");
     }
     return 0;
 }
 
 
+/*--------------------------------------------------------------------*
+ *                         Conversion to pdf                          *
+ *--------------------------------------------------------------------*/
+/*!
+ *  pixacompConvertToPdf()
+ *
+ *      Input:  pixac (containing images all at the same resolution)
+ *              res (override the resolution of each input image, in ppi;
+ *                   use 0 to respect the resolution embedded in the input)
+ *              scalefactor (scaling factor applied to each image; > 0.0)
+ *              type (encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
+ *                    L_FLATE_ENCODE, or 0 for default)
+ *              quality (used for JPEG only; 0 for default (75))
+ *              title (<optional> pdf title; if null, taken from the first
+ *                     image filename)
+ *              fileout (pdf file of all images)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This follows closely the function pixaConvertToPdf() in pdfio.c.
+ *      (2) The images are encoded with G4 if 1 bpp; JPEG if 8 bpp without
+ *          colormap and many colors, or 32 bpp; FLATE for anything else.
+ *      (3) The scalefactor must be > 0.0; otherwise it is set to 1.0.
+ *      (4) Specifying one of the three encoding types for @type forces
+ *          all images to be compressed with that type.  Use 0 to have
+ *          the type determined for each image based on depth and whether
+ *          or not it has a colormap.
+ */
+l_int32
+pixacompConvertToPdf(PIXAC       *pixac,
+                     l_int32      res,
+                     l_float32    scalefactor,
+                     l_int32      type,
+                     l_int32      quality,
+                     const char  *title,
+                     const char  *fileout)
+{
+l_uint8  *data;
+l_int32   ret;
+size_t    nbytes;
+
+    PROCNAME("pixacompConvertToPdf");
+
+    if (!pixac)
+        return ERROR_INT("pixac not defined", procName, 1);
+
+    ret = pixacompConvertToPdfData(pixac, res, scalefactor, type, quality,
+                                   title, &data, &nbytes);
+    if (ret) {
+        FREE(data);
+        return ERROR_INT("conversion to pdf failed", procName, 1);
+    }
+
+    ret = l_binaryWrite(fileout, "w", data, nbytes);
+    FREE(data);
+    if (ret)
+        L_ERROR("pdf data not written to file", procName);
+    return ret;
+}
+
+
+/*!
+ *  pixacompConvertToPdfData()
+ *
+ *      Input:  pixac (containing images all at the same resolution)
+ *              res (input resolution of all images)
+ *              scalefactor (scaling factor applied to each image; > 0.0)
+ *              type (encoding type (L_JPEG_ENCODE, L_G4_ENCODE,
+ *                    L_FLATE_ENCODE, or 0 for default)
+ *              quality (used for JPEG only; 0 for default (75))
+ *              title (<optional> pdf title; if null, taken from the first
+ *                     image filename)
+ *              &data (<return> output pdf data (of all images)
+ *              &nbytes (<return> size of output pdf data)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) See pixacompConvertToPdf().
+ */
+l_int32
+pixacompConvertToPdfData(PIXAC       *pixac,
+                         l_int32      res,
+                         l_float32    scalefactor,
+                         l_int32      type,
+                         l_int32      quality,
+                         const char  *title,
+                         l_uint8    **pdata,
+                         size_t      *pnbytes)
+{
+l_uint8  *imdata;
+l_int32   i, n, ret, scaledres, pagetype;
+size_t    imbytes;
+L_BYTEA  *ba;
+PIX      *pixs, *pix;
+L_PTRA   *pa_data;
+
+    PROCNAME("pixacompConvertToPdfData");
+
+    if (!pdata)
+        return ERROR_INT("&data not defined", procName, 1);
+    *pdata = NULL;
+    if (!pnbytes)
+        return ERROR_INT("&nbytes not defined", procName, 1);
+    *pnbytes = 0;
+    if (!pixac)
+        return ERROR_INT("pixac not defined", procName, 1);
+    if (scalefactor <= 0.0) scalefactor = 1.0;
+    if (type < 0 || type > L_FLATE_ENCODE) {
+        L_WARNING("invalid compression type; using per-page default", procName);
+        type = 0;
+    }
+
+        /* Generate all the encoded pdf strings */
+    n = pixacompGetCount(pixac);
+    pa_data = ptraCreate(n);
+    for (i = 0; i < n; i++) {
+        if ((pixs =
+             pixacompGetPix(pixac, pixacompGetOffset(pixac) + i)) == NULL) {
+            L_ERROR_INT("pix[%d] not retrieved", procName, i);
+            continue;
+        }
+        if (pixGetWidth(pixs) == 1) {  /* used sometimes as placeholders */
+            L_INFO_INT("placeholder image[%d] has w = 1", procName, i);
+            pixDestroy(&pixs);
+            continue;
+        }
+        if (scalefactor != 1.0)
+            pix = pixScale(pixs, scalefactor, scalefactor);
+        else
+            pix = pixClone(pixs);
+        pixDestroy(&pixs);
+        scaledres = (l_int32)(res * scalefactor);
+        if (type != 0)
+            pagetype = type;
+        else if (selectDefaultPdfEncoding(pix, &pagetype) != 0) {
+            L_ERROR_INT("encoding type selection failed for pix[%d]",
+                        procName, i);
+            pixDestroy(&pix);
+            continue;
+        }
+        ret = pixConvertToPdfData(pix, pagetype, quality, &imdata, &imbytes,
+                                  0, 0, scaledres, NULL, 0, title);
+        pixDestroy(&pix);
+        if (ret) {
+            L_ERROR_INT("pdf encoding failed for pix[%d]", procName, i);
+            continue;
+        }
+        ba = l_byteaInitFromMem(imdata, imbytes);
+        if (imdata) FREE(imdata);
+        ptraAdd(pa_data, ba);
+    }
+    ptraGetActualCount(pa_data, &n);
+    if (n == 0) {
+        L_ERROR("no pdf files made", procName);
+        ptraDestroy(&pa_data, FALSE, FALSE);
+        return 1;
+    }
+
+        /* Concatenate them */
+    ret = ptraConcatenatePdfToData(pa_data, NULL, pdata, pnbytes);
+
+    ptraGetActualCount(pa_data, &n);  /* recalculate in case it changes */
+    for (i = 0; i < n; i++) {
+        ba = (L_BYTEA *)ptraRemove(pa_data, i, L_NO_COMPACTION);
+        l_byteaDestroy(&ba);
+    }
+    ptraDestroy(&pa_data, FALSE, FALSE);
+    return ret;
+}
+
+
 /*--------------------------------------------------------------------*
  *                        Output for debugging                        *
  *--------------------------------------------------------------------*/
@@ -1381,12 +1675,13 @@ PIXC    *pixc;
     nboxes = pixacompGetBoxaCount(pixac);
     fprintf(fp, "Number of pixcomp: %d\n", n);
     fprintf(fp, "Size of pixcomp array alloc: %d\n", pixac->nalloc);
+    fprintf(fp, "Offset of index into array: %d\n", pixac->offset);
     if (nboxes  > 0)
         fprintf(fp, "Boxa has %d boxes\n", nboxes);
     else
         fprintf(fp, "Boxa is empty\n");
     for (i = 0; i < n; i++) {
-        pixc = pixacompGetPixcomp(pixac, i);
+        pixc = pixacompGetPixcomp(pixac, pixac->offset + i);
         pixcompWriteStreamInfo(fp, pixc, NULL);
     }
     return 0;
@@ -1483,7 +1778,7 @@ PIXA      *pixan;
         return (PIX *)ERROR_PTR("outdepth not in {1, 8, 32}", procName, NULL);
     if (border < 0 || border > tilewidth / 5)
         border = 0;
-    
+
     if ((n = pixacompGetCount(pixac)) == 0)
         return (PIX *)ERROR_PTR("no components", procName, NULL);
 
@@ -1491,7 +1786,8 @@ PIXA      *pixan;
     pixan = pixaCreate(n);
     bordval = (outdepth == 1) ? 1 : 0;
     for (i = 0; i < n; i++) {
-        if ((pix = pixacompGetPix(pixac, i)) == NULL) {
+        if ((pix =
+             pixacompGetPix(pixac, pixacompGetOffset(pixac) + i)) == NULL) {
             L_WARNING_INT("pix %d not made", procName, i);
             continue;
         }
@@ -1579,5 +1875,3 @@ PIXA      *pixan;
     FREE(rowht);
     return pixd;
 }
-
-
