@@ -103,9 +103,10 @@
 #endif  /* HAVE_CONFIG_H */
 
 /* --------------------------------------------*/
-#if  HAVE_LIBPNG   /* defined in environ.h */
+#if  HAVE_LIBPNG && HAVE_LIBZ   /* defined in environ.h */
 /* --------------------------------------------*/
 
+#include "zlib.h"
 #include "png.h"
 
 /* ----------------Set defaults for read/write options ----------------- */
@@ -226,6 +227,7 @@ PIXCMAP     *cmap;
         /* Remove if/when this is implemented for all bit_depths */
     if (spp == 3 && bit_depth != 8) {
         fprintf(stderr, "Help: spp = 3 and depth = %d != 8\n!!", bit_depth);
+        png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
         return (PIX *)ERROR_PTR("not implemented for this depth",
             procName, NULL);
     }
@@ -288,7 +290,7 @@ PIXCMAP     *cmap;
         /* If there is no colormap, PNG defines black = 0 and
          * white = 1 by default for binary monochrome.  Therefore,
          * since we use the opposite definition, we must invert
-         * the image in either of these cases:
+         * the image colors in either of these cases:
          *    (i) there is no colormap (default)
          *    (ii) there is a colormap which defines black to
          *         be 0 and white to be 1.
@@ -307,10 +309,17 @@ PIXCMAP     *cmap;
          *     if black = 0, white = 1 (255)
          *          0, 0, 0, 0, 255, 255, 255, 0
          * So we test the first byte to see if it is 0;
-         * if so, invert the data.  */
+         * if so, invert the colors.
+         *
+         * If there is a colormap, after inverting the pixels it is
+         * necessary to destroy the colormap.  Otherwise, if someone were
+         * to call pixRemoveColormap(), this would cause the pixel
+         * values to be inverted again!
+         */
     if (d == 1 && (!cmap || (cmap && ((l_uint8 *)(cmap->array))[0] == 0x0))) {
 /*        fprintf(stderr, "Inverting binary data on png read\n"); */
         pixInvert(pix, pix);
+        pixDestroyColormap(pix);
     }
 
     xres = png_get_x_pixels_per_meter(png_ptr, info_ptr);
@@ -1140,6 +1149,5 @@ pixWriteMemPng(l_uint8  **pdata,
 #endif  /* HAVE_FMEMOPEN */
 
 /* --------------------------------------------*/
-#endif  /* HAVE_LIBPNG */
+#endif  /* HAVE_LIBPNG && HAVE_LIBZ */
 /* --------------------------------------------*/
-
