@@ -20,6 +20,7 @@
  *       Regression test utilities
  *           l_int32    regTestSetup()
  *           l_int32    regTestCleanup()
+ *           l_int32    regTestCompareValues()
  *           l_int32    regTestComparePix()
  *           l_int32    regTestCompareSimilarPix()
  *           l_int32    regTestCheckFile()
@@ -33,7 +34,6 @@
 #include <string.h>
 #include "allheaders.h"
 
-
 extern l_int32 NumImageFileFormatExtensions;
 extern const char *ImageFileFormatExtensions[];
 
@@ -161,10 +161,11 @@ L_REGPARAMS  *rp;
 l_int32
 regTestCleanup(L_REGPARAMS  *rp)
 {
-char    result[512];
-char   *results_file;  /* success/failure output in 'compare' mode */
-char   *text, *message;
-size_t  nbytes;
+char     result[512];
+char    *results_file;  /* success/failure output in 'compare' mode */
+char    *text, *message;
+l_int32  retval;
+size_t   nbytes;
 
     PROCNAME("regTestCleanup");
 
@@ -201,11 +202,55 @@ size_t  nbytes;
     FREE(text);
     results_file = genPathname("/tmp", "reg_results.txt");
     fileAppendString(results_file, message);
+    retval = (rp->success) ? 0 : 1;
     FREE(results_file);
     FREE(message);
 
     FREE(rp->testname);
     FREE(rp);
+    return retval;
+}
+
+
+/*!
+ *  regTestCompareValues()
+ *
+ *      Input:  rp (regtest parameters)
+ *              val1 (typ. the golden value)
+ *              val2 (typ. the value computed)
+ *              delta (allowed max absolute difference)
+ *      Return: 0 if OK, 1 on error (a failure in comparison is not an error)
+ */
+l_int32
+regTestCompareValues(L_REGPARAMS  *rp,
+                     l_float32     val1,
+                     l_float32     val2,
+                     l_float32     delta)
+{
+l_float32  diff;
+
+    PROCNAME("regTestCompareValues");
+
+    if (!rp)
+        return ERROR_INT("rp not defined", procName, 1);
+
+    rp->index++;
+    diff = L_ABS(val2 - val1);
+
+        /* Record on failure */
+    if (diff > delta) {
+        if (rp->fp) {
+            fprintf(rp->fp,
+                    "Failure in %s_reg: value comparison for index %d\n"
+                    "difference = %f but allowed delta = %f\n",
+                    rp->testname, rp->index, diff, delta);
+        }
+        fprintf(stderr,
+                    "Failure in %s_reg: value comparison for index %d\n"
+                    "difference = %f but allowed delta = %f\n",
+                    rp->testname, rp->index, diff, delta);
+        rp->success = FALSE;
+    }
     return 0;
 }
 
@@ -574,5 +619,3 @@ char    *root;
     root[len - 4] = '\0';  /* remove the suffix */
     return root;
 }
-
-
