@@ -107,14 +107,14 @@ PIXCMAP   *cmap;
 
         /* Don't rotate by shear more than 20 degrees */
     if (L_ABS(angle) > 0.35 && type == L_ROTATE_SHEAR) {
-        L_WARNING("large angle; rotating by sampling", procName);
+        L_INFO("large angle; rotating by sampling", procName);
         type = L_ROTATE_SAMPLING;
     }
 
         /* If 1 bpp and area map is requested, rotate by sampling */
     d = pixGetDepth(pixs);
     if (d == 1 && type == L_ROTATE_AREA_MAP) {
-        L_WARNING("1 bpp; rotating by sampling", procName);
+        L_INFO("1 bpp; rotating by sampling", procName);
         type = L_ROTATE_SAMPLING;
     }
 
@@ -588,4 +588,3 @@ PIX  *pixg, *pixd;
     pixDestroy(&pixg);
     return pixd;
 }
-
