@@ -76,6 +76,7 @@
  *      Directory filenames
  *          SARRAY    *getNumberedPathnamesInDirectory()
  *          SARRAY    *getSortedPathnamesInDirectory()
+ *          SARRAY    *convertSortedToNumberedPathnames()
  *          SARRAY    *getFilenamesInDirectory()
  *
  *      Comments on usage:
@@ -1597,11 +1598,10 @@ FILE  *fp;
  *      Input:  directory name
  *              substr (<optional> substring filter on filenames; can be NULL)
  *              numpre (number of characters in name before number)
- *              numpost (number of characters in name after number, up
- *                       to a dot before an extension)
- *                       including an extension and the dot separator)
+ *              numpost (number of characters in name after the number,
+ *                       up to a dot before an extension)
  *              maxnum (only consider page numbers up to this value)
- *      Return: sarray of sorted pathnames, or NULL on error
+ *      Return: sarray of numbered pathnames, or NULL on error
  *
  *  Notes:
  *      (1) Returns the full pathnames of the numbered filenames in
@@ -1622,7 +1622,9 @@ FILE  *fp;
  *          preceeding the actual page number; @numpost is the number
  *          following the page number, up to either the end of the
  *          basename or a ".", whichever comes first.
- *      (5) To use a O(n) matching algorithm, the largest page number
+ *      (5) This is useful when all filenames contain numbers that are
+ *          not necessarily consecutive.  0-padding is not required.
+ *      (6) To use a O(n) matching algorithm, the largest page number
  *          is found and two internal arrays of this size are created.
  *          This maximum is constrained not to exceed @maxsum,
  *          to make sure that an unrealistically large number is not
@@ -1635,8 +1637,7 @@ getNumberedPathnamesInDirectory(const char  *dirname,
                                 l_int32      numpost,
                                 l_int32      maxnum)
 {
-char    *fname, *str;
-l_int32  i, nfiles, num, index;
+l_int32  nfiles;
 SARRAY  *sa, *saout;
 
     PROCNAME("getNumberedPathnamesInDirectory");
@@ -1649,36 +1650,7 @@ SARRAY  *sa, *saout;
     if ((nfiles = sarrayGetCount(sa)) == 0)
         return sarrayCreate(1);
 
-        /* Find the last file in the sorted array that has a number
-         * that (a) matches the count pattern and (b) does not
-         * exceed @maxnum.  @maxnum sets an upper limit on the size
-         * of the sarray.  */
-    num = 0;
-    for (i = nfiles - 1; i >= 0; i--) {
-      fname = sarrayGetString(sa, i, L_NOCOPY);
-      num = extractNumberFromFilename(fname, numpre, numpost);
-      if (num < 0) continue;
-      num = L_MIN(num + 1, maxnum);
-      break;
-    }
-
-    if (num <= 0)  /* none found */
-        return sarrayCreate(1);
-
-        /* Insert pathnames into the output sarray.
-         * Ignore numbers that are out of the range of sarray. */
-    saout = sarrayCreateInitialized(num, (char *)"");
-    for (i = 0; i < nfiles; i++) {
-      fname = sarrayGetString(sa, i, L_NOCOPY);
-      index = extractNumberFromFilename(fname, numpre, numpost);
-      if (index < 0 || index >= num) continue;
-      str = sarrayGetString(saout, index, L_NOCOPY);
-      if (str[0] != '\0')
-          L_WARNING_INT("\n  Multiple files with same number: %d",
-                        procName, index);
-      sarrayReplaceString(saout, index, fname, L_COPY);
-    }
-
+    saout = convertSortedToNumberedPathnames(sa, numpre, numpost, maxnum);
     sarrayDestroy(&sa);
     return saout;
 }
@@ -1746,6 +1718,71 @@ SARRAY  *sa, *safiles, *saout;
 }
 
 
+/*!
+ *  convertSortedToNumberedPathnames()
+ *
+ *      Input:  sorted pathnames (including zero-padded integers)
+ *              numpre (number of characters in name before number)
+ *              numpost (number of characters in name after the number,
+ *                       up to a dot before an extension)
+ *              maxnum (only consider page numbers up to this value)
+ *      Return: sarray of numbered pathnames, or NULL on error
+ *
+ *  Notes:
+ *      (1) Typically, numpre = numpost = 0; e.g., when the filename
+ *          just has a number followed by an optional extension.
+ */
+SARRAY *
+convertSortedToNumberedPathnames(SARRAY   *sa,
+                                 l_int32   numpre,
+                                 l_int32   numpost,
+                                 l_int32   maxnum)
+{
+char    *fname, *str;
+l_int32  i, nfiles, num, index;
+SARRAY  *saout;
+
+    PROCNAME("convertSortedToNumberedPathnames");
+
+    if (!sa)
+        return (SARRAY *)ERROR_PTR("sa not defined", procName, NULL);
+    if ((nfiles = sarrayGetCount(sa)) == 0)
+        return sarrayCreate(1);
+
+        /* Find the last file in the sorted array that has a number
+         * that (a) matches the count pattern and (b) does not
+         * exceed @maxnum.  @maxnum sets an upper limit on the size
+         * of the sarray.  */
+    num = 0;
+    for (i = nfiles - 1; i >= 0; i--) {
+      fname = sarrayGetString(sa, i, L_NOCOPY);
+      num = extractNumberFromFilename(fname, numpre, numpost);
+      if (num < 0) continue;
+      num = L_MIN(num + 1, maxnum);
+      break;
+    }
+
+    if (num <= 0)  /* none found */
+        return sarrayCreate(1);
+
+        /* Insert pathnames into the output sarray.
+         * Ignore numbers that are out of the range of sarray. */
+    saout = sarrayCreateInitialized(num, (char *)"");
+    for (i = 0; i < nfiles; i++) {
+      fname = sarrayGetString(sa, i, L_NOCOPY);
+      index = extractNumberFromFilename(fname, numpre, numpost);
+      if (index < 0 || index >= num) continue;
+      str = sarrayGetString(saout, index, L_NOCOPY);
+      if (str[0] != '\0')
+          L_WARNING_INT("\n  Multiple files with same number: %d",
+                        procName, index);
+      sarrayReplaceString(saout, index, fname, L_COPY);
+    }
+
+    return saout;
+}
+
+
 /*!
  *  getFilenamesInDirectory()
  *
@@ -1866,4 +1903,3 @@ WIN32_FIND_DATAA  ffd;
 }
 
 #endif  /* _WIN32 */
-
