@@ -207,7 +207,7 @@ FILE  *fp;
 
 #if  WRITE_AS_NAMED  /* Default */
 
-    if ((fp = fopen(fname, "wb+")) == NULL) {
+    if ((fp = fopenWriteStream(fname, "wb+")) == NULL) {
         FREE(fname);
         return ERROR_INT("stream not opened", procName, 1);
     }
@@ -235,7 +235,7 @@ FILE  *fp;
         else
             filebuf = (char *)fname;
 
-        fp = fopen(filebuf, "wb+");
+        fp = fopenWriteStream(filebuf, "wb+");
         if (filebuf != fname)
             FREE(filebuf);
         if (fp == NULL) {
